// ============================================================================
// Copyright (c) 2024 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  No.80, Fenggong Rd., Hukou Township, Hsinchu County 303035. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Fri Apr 12 10:01:00 2024
// ============================================================================

`define ENABLE_DDR4A
`define ENABLE_DDR4B
//`define ENABLE_QSFP
//`define ENABLE_FMCP
//`define ENABLE_HPS
//`define ENABLE_HDMI
//`define ENABLE_PCIE
//`define ENABLE_ENET
//`define ENABLE_CAM1
//`define ENABLE_CAM2

module golden_top(

      ///////// CLOCK /////////
      input              CLK_100_B2B_p,
      input              CLK_50_B5A,
      input              CLK_50_B6A,
      input              CLK_50_B6C,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 3: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 3: 0]   SW,

      ///////// LED /////////
      output   [ 3: 0]   LED, //LED is Low-Active

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 0: 0]   DDR4A_BG,
      output             DDR4A_CK,
      output             DDR4A_CK_n,
      output             DDR4A_CKE,
      inout    [ 3: 0]   DDR4A_DQS,
      inout    [ 3: 0]   DDR4A_DQS_n,
      inout    [31: 0]   DDR4A_DQ,
      inout    [ 3: 0]   DDR4A_DBI_n,
      output             DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output             DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 0: 0]   DDR4B_BG,
      output             DDR4B_CK,
      output             DDR4B_CK_n,
      output             DDR4B_CKE,
      inout    [ 3: 0]   DDR4B_DQS,
      inout    [ 3: 0]   DDR4B_DQS_n,
      inout    [31: 0]   DDR4B_DQ,
      inout    [ 3: 0]   DDR4B_DBI_n,
      output             DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output             DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_HDMI
      ///////// HDMI /////////
      inout              HDMI_LRCLK,
      inout              HDMI_MCLK,
      inout              HDMI_SCLK,
      output             HDMI_TX_CLK,
      output             HDMI_TX_HS,
      output             HDMI_TX_VS,
      output   [23: 0]   HDMI_TX_D,
      output             HDMI_TX_DE,
      inout              HDMI_I2C_SCL,
      inout              HDMI_I2C_SDA,
      input              HDMI_TX_INT,
      inout              HDMI_I2S,
`endif /*ENABLE_HDMI*/

`ifdef ENABLE_ENET
      ///////// ENET /////////
      output             ENET_88E2110_TX_p,
      output             ENET_88E2110_TX_n,
      input              ENET_88E2110_RX_p,
      input              ENET_88E2110_RX_n,
      input              ENET_88E2110_REFCLK_125M_p,
      input              ENET_88E2110_INT_n,
      output             ENET_88E2110_MDC,
      inout              ENET_88E2110_MDIO,
      output             ENET_88E2110_RESET_n,
`endif /*ENABLE_ENET*/

      ///////// SI5340B /////////
      inout              SI5340B_I2C_SCL,
      inout              SI5340B_I2C_SDA,
      output             SI5340B_OE_n,
      output             SI5340B_RST_n,

      ///////// SI5391B /////////
      inout              SI5391B_I2C_SCL,
      inout              SI5391B_I2C_SDA,
      output             SI5391B_OE_n,
      output             SI5391B_RST_n,

      ///////// GPIO /////////
      inout    [35: 0]   GPIO,

      ///////// TMD /////////
      inout    [ 7: 0]   TMD_D,

`ifdef ENABLE_FMCP
      ///////// FMCP /////////
      inout              FMCP_CLK2_BIDIR_p,
      inout              FMCP_CLK2_BIDIR_n,
      inout              FMCP_CLK3_BIDIR_p,
      inout              FMCP_CLK3_BIDIR_n,
      output   [11: 0]   FMCP_DP_C2M_p,
      output   [11: 0]   FMCP_DP_C2M_n,
      input    [11: 0]   FMCP_DP_M2C_p,
      input    [11: 0]   FMCP_DP_M2C_n,
      inout    [23: 0]   FMCP_HA_p,
      inout    [23: 0]   FMCP_HA_n,
      inout    [21: 0]   FMCP_HB_p,
      inout    [21: 0]   FMCP_HB_n,
      inout    [33: 0]   FMCP_LA_p,
      inout    [33: 0]   FMCP_LA_n,
      input    [ 1: 0]   FMCP_CLK_M2C_p,
      input    [ 2: 0]   FMCP_GBTCLK_M2C_p,
      output             FMCP_REFCLK_C2M_p,
      input              FMCP_REFCLK_M2C_p,
      input    [ 2: 0]   FMCP_REFCLK_p,
      input              FMCP_RES0,
      inout              FMCP_SCL,
      inout              FMCP_SDA,
      output             FMCP_SYNC_C2M_p,
      input              FMCP_SYNC_M2C_p,
`endif /*ENABLE_FMCP*/

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      output   [ 3: 0]   PCIE_TX_p,
      output   [ 3: 0]   PCIE_TX_n,
      input    [ 3: 0]   PCIE_RX_p,
      input    [ 3: 0]   PCIE_RX_n,
      input              PCIE_OB_REFCLK_p,
      input              PCIE_REFCLK_p,
      input              PCIE_PERST_n,
      output             PCIE_WAKE_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFP
      ///////// QSFP /////////
      output   [ 3: 0]   QSFP_TX_p,
      output   [ 3: 0]   QSFP_TX_n,
      input    [ 3: 0]   QSFP_RX_p,
      input    [ 3: 0]   QSFP_RX_n,
      input              CIPRI_REFCLK_p,
      input              QSFP_REFCLK_p,
      input              QSFP_INTERRUPT_n,
      output             QSFP_LP_MODE,
      input              QSFP_MOD_PRS_n,
      output             QSFP_MOD_SEL_n,
      output             QSFP_RST_n,
      inout              QSFP_SCL,
      inout              QSFP_SDA,
`endif /*ENABLE_QSFP*/

      ///////// CAM /////////
      input              CAM_RZQ0,

`ifdef ENABLE_CAM1
      ///////// CAM1 /////////
      input              CAM1_CLK_p,
      input              CAM1_CLK_n,
      input    [ 1: 0]   CAM1_D_p,
      input    [ 1: 0]   CAM1_D_n,
      inout              CAM1_I2C_SCL,
      inout              CAM1_I2C_SDA,
      inout              CAM1_GPIO,
`endif /*ENABLE_CAM1*/

`ifdef ENABLE_CAM2
      ///////// CAM2 /////////
      input              CAM2_CLK_p,
      input              CAM2_CLK_n,
      input    [ 1: 0]   CAM2_D_p,
      input    [ 1: 0]   CAM2_D_n,
      inout              CAM2_I2C_SCL,
      inout              CAM2_I2C_SDA,
      inout              CAM2_GPIO,
`endif /*ENABLE_CAM2*/

`ifdef ENABLE_HPS
      ///////// HPS /////////
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      inout    [ 4: 0]   HPS_GPIO,
      inout              HPS_I2C_SCL,
      inout              HPS_I2C_SDA,
      inout              HPS_KEY,
      inout              HPS_LED,
      input              HPS_OSC_CLK,
      output             HPS_SDMMC_CLK,
      inout              HPS_SDMMC_CMD,
      inout    [ 7: 0]   HPS_SDMMC_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
      input              HPS_USB3_REFCLK_100M_p,
      input              HPS_USB3_SS_RX_n,
      input              HPS_USB3_SS_RX_p,
      output             HPS_USB3_SS_TX_n,
      output             HPS_USB3_SS_TX_p,
      input              HPS_USB_CLK,
      inout    [ 7: 0]   HPS_USB_DATA,
      input              HPS_USB_DIR,
      input              HPS_USB_ID,
      input              HPS_USB_NXT,
      output             HPS_USB_STP,
      output             HPS_USB_VBUS_CTRL,
      input              HPS_USB_VBUS_DET,
      input              HPS_USB_VBUS_FLT_n,
`endif /*ENABLE_HPS*/

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n

);


//=======================================================
//  REG/WIRE declarations
//=======================================================

assign SI5340B_OE_n = 1'bZ;
assign SI5340B_RST_n = 1'bZ;
assign SI5391B_OE_n = 1'bZ;
assign SI5391B_RST_n = 1'bZ;




//=======================================================
//  Structural coding
//=======================================================
wire emif_rst_n_ddr4a;
wire emif_rst_n_ddr4b;
wire DDR4_IOPLL_Lock;
wire reset_handle;

// test status ..
//DDR4 Verify (A)
wire ddr4a_test_pass/*synthesis keep*/;
wire ddr4a_test_fail/*synthesis keep*/;
wire ddr4a_test_complete/*synthesis keep*/;
wire ddr4a_local_cal_success/*synthesis keep*/;
wire ddr4a_local_cal_fail/*synthesis keep*/;
wire ddr4a_reset_done/*synthesis keep*/;
//DDR4 Verify (B)
wire ddr4b_test_pass/*synthesis keep*/;
wire ddr4b_test_fail/*synthesis keep*/;
wire ddr4b_test_complete/*synthesis keep*/;
wire ddr4b_local_cal_success/*synthesis keep*/;
wire ddr4b_local_cal_fail/*synthesis keep*/;
wire ddr4b_reset_done/*synthesis keep*/;
 

wire AXI_LITE_CLK;
parameter AXIL_DRIVER_ADDRESS_WIDTH = 27;  
assign reset_handle = DDR4_IOPLL_Lock & !ninit_done;
	 
	 
wire global_reset;
wire	ninit_done;

debouncer debouncer_Inst(
    .clk(CLK_50_B5A),         // clock signal
    .reset(ninit_done),       // reset signal
    .button_in(BUTTON[1]),   // raw button input
    .button_out(global_reset)   // debounced button output
);

wire ninit_done;
    reset_release reset_release_inst (
        .ninit_done (ninit_done)  //  output,  width = 1, ninit_done.ninit_done
    );
		

wire	aclk;	 
wire	aresetn;
wire	pipe_aresetn;
assign pipe_aresetn = global_reset & emif_rst_n_ddr4a & emif_rst_n_ddr4b & DDR4A_cal_done_rst_n & DDR4B_cal_done_rst_n;

 

    EMIF_Qsys  EMIF_Qsys_Inst(
        .clock_310m_out_clk_clk                   (aclk),                   //  output,    width = 1,               clock_310m_out_clk.clk
        .emif_ddr4a_s0_axi4_ctrl_ready_reset_n  (emif_rst_n_ddr4a),  //  output,    width = 1,  emif_ddr4a_s0_axi4_ctrl_ready.reset_n
        .emif_ddr4a_s0_axi4_awaddr              (DDR4A_awaddr),              //   input,   width = 32,             emif_ddr4a_s0_axi4.awaddr
        .emif_ddr4a_s0_axi4_awburst             (DDR4A_awburst),             //   input,    width = 2,                                 .awburst
        .emif_ddr4a_s0_axi4_awid                (DDR4A_awid),                //   input,    width = 7,                                 .awid
        .emif_ddr4a_s0_axi4_awlen               (DDR4A_awlen),               //   input,    width = 8,                                 .awlen
        .emif_ddr4a_s0_axi4_awlock              (DDR4A_awlock),              //   input,    width = 1,                                 .awlock
        .emif_ddr4a_s0_axi4_awqos               (0),               //   input,    width = 4,                                 .awqos
        .emif_ddr4a_s0_axi4_awsize              (DDR4A_awsize),              //   input,    width = 3,                                 .awsize
        .emif_ddr4a_s0_axi4_awvalid             (DDR4A_awvalid),             //   input,    width = 1,                                 .awvalid
        .emif_ddr4a_s0_axi4_awuser              (0),              //   input,   width = 14,                                 .awuser
        .emif_ddr4a_s0_axi4_awprot              (DDR4A_awprot),              //   input,    width = 3,                                 .awprot
        .emif_ddr4a_s0_axi4_awready             (DDR4A_awready),             //  output,    width = 1,                                 .awready
        .emif_ddr4a_s0_axi4_araddr              (DDR4A_araddr),              //   input,   width = 32,                                 .araddr
        .emif_ddr4a_s0_axi4_arburst             (DDR4A_arburst),             //   input,    width = 2,                                 .arburst
        .emif_ddr4a_s0_axi4_arid                (DDR4A_arid),                //   input,    width = 7,                                 .arid
        .emif_ddr4a_s0_axi4_arlen               (DDR4A_arlen),               //   input,    width = 8,                                 .arlen
        .emif_ddr4a_s0_axi4_arlock              (DDR4A_arlock),              //   input,    width = 1,                                 .arlock
        .emif_ddr4a_s0_axi4_arqos               (0),               //   input,    width = 4,                                 .arqos
        .emif_ddr4a_s0_axi4_arsize              (DDR4A_arsize),              //   input,    width = 3,                                 .arsize
        .emif_ddr4a_s0_axi4_arvalid             (DDR4A_arvalid),             //   input,    width = 1,                                 .arvalid
        .emif_ddr4a_s0_axi4_aruser              (0),              //   input,   width = 14,                                 .aruser
        .emif_ddr4a_s0_axi4_arprot              (DDR4A_arprot),              //   input,    width = 3,                                 .arprot
        .emif_ddr4a_s0_axi4_arready             (DDR4A_arready),             //  output,    width = 1,                                 .arready
        .emif_ddr4a_s0_axi4_wdata               (DDR4A_wdata),               //   input,  width = 256,                                 .wdata
        .emif_ddr4a_s0_axi4_wstrb               (DDR4A_wstrb),               //   input,   width = 32,                                 .wstrb
        .emif_ddr4a_s0_axi4_wlast               (DDR4A_wlast),               //   input,    width = 1,                                 .wlast
        .emif_ddr4a_s0_axi4_wvalid              (DDR4A_wvalid),              //   input,    width = 1,                                 .wvalid
        .emif_ddr4a_s0_axi4_wuser               (0),               //   input,   width = 32,                                 .wuser
        .emif_ddr4a_s0_axi4_wready              (DDR4A_wready),              //  output,    width = 1,                                 .wready
        .emif_ddr4a_s0_axi4_bready              (DDR4A_bready),              //   input,    width = 1,                                 .bready
        .emif_ddr4a_s0_axi4_bid                 (DDR4A_bid),                 //  output,    width = 7,                                 .bid
        .emif_ddr4a_s0_axi4_bresp               (DDR4A_bresp),               //  output,    width = 2,                                 .bresp
        .emif_ddr4a_s0_axi4_bvalid              (DDR4A_bvalid),              //  output,    width = 1,                                 .bvalid
        .emif_ddr4a_s0_axi4_rready              (DDR4A_rready),              //   input,    width = 1,                                 .rready
        .emif_ddr4a_s0_axi4_ruser               (),               //  output,   width = 32,                                 .ruser
        .emif_ddr4a_s0_axi4_rdata               (DDR4A_rdata),               //  output,  width = 256,                                 .rdata
        .emif_ddr4a_s0_axi4_rid                 (DDR4A_rid),                 //  output,    width = 7,                                 .rid
        .emif_ddr4a_s0_axi4_rlast               (DDR4A_rlast),               //  output,    width = 1,                                 .rlast
        .emif_ddr4a_s0_axi4_rresp               (DDR4A_rresp),               //  output,    width = 2,                                 .rresp
        .emif_ddr4a_s0_axi4_rvalid              (DDR4A_rvalid),              //  output,    width = 1,                                 .rvalid
        .emif_ddr4a_s0_axi4lite_clock_clk       (AXI_LITE_CLK),       //   input,    width = 1,   emif_ddr4a_s0_axi4lite_clock.clk
        .emif_ddr4a_s0_axi4lite_reset_n_reset_n (reset_handle), //   input,    width = 1, emif_ddr4a_s0_axi4lite_reset_n.reset_n
        .emif_ddr4a_s0_axi4lite_awaddr          (DDR4A_axil_driver_awaddr),          //   input,   width = 27,         emif_ddr4a_s0_axi4lite.awaddr
        .emif_ddr4a_s0_axi4lite_awprot          (DDR4A_axil_driver_awprot),          //   input,    width = 3,                                 .awprot
        .emif_ddr4a_s0_axi4lite_awvalid         (DDR4A_axil_driver_awvalid),         //   input,    width = 1,                                 .awvalid
        .emif_ddr4a_s0_axi4lite_awready         (DDR4A_axil_driver_awready),         //  output,    width = 1,                                 .awready
        .emif_ddr4a_s0_axi4lite_araddr          (DDR4A_axil_driver_araddr),          //   input,   width = 27,                                 .araddr
        .emif_ddr4a_s0_axi4lite_arprot          (DDR4A_axil_driver_arprot),          //   input,    width = 3,                                 .arprot
        .emif_ddr4a_s0_axi4lite_arvalid         (DDR4A_axil_driver_arvalid),         //   input,    width = 1,                                 .arvalid
        .emif_ddr4a_s0_axi4lite_arready         (DDR4A_axil_driver_arready),         //  output,    width = 1,                                 .arready
        .emif_ddr4a_s0_axi4lite_wdata           (DDR4A_axil_driver_wdata),           //   input,   width = 32,                                 .wdata
        .emif_ddr4a_s0_axi4lite_wstrb           (DDR4A_axil_driver_wstrb),           //   input,    width = 4,                                 .wstrb
        .emif_ddr4a_s0_axi4lite_wvalid          (DDR4A_axil_driver_wvalid),          //   input,    width = 1,                                 .wvalid
        .emif_ddr4a_s0_axi4lite_wready          (DDR4A_axil_driver_wready),          //  output,    width = 1,                                 .wready
        .emif_ddr4a_s0_axi4lite_bready          (DDR4A_axil_driver_bready),          //   input,    width = 1,                                 .bready
        .emif_ddr4a_s0_axi4lite_bresp           (DDR4A_axil_driver_bresp),           //  output,    width = 2,                                 .bresp
        .emif_ddr4a_s0_axi4lite_bvalid          (DDR4A_axil_driver_bvalid),          //  output,    width = 1,                                 .bvalid
        .emif_ddr4a_s0_axi4lite_rready          (DDR4A_axil_driver_rready),          //   input,    width = 1,                                 .rready
        .emif_ddr4a_s0_axi4lite_rdata           (DDR4A_axil_driver_rdata),           //  output,   width = 32,                                 .rdata
        .emif_ddr4a_s0_axi4lite_rresp           (DDR4A_axil_driver_rresp),           //  output,    width = 2,                                 .rresp
        .emif_ddr4a_s0_axi4lite_rvalid          (DDR4A_axil_driver_rvalid),          //  output,    width = 1,                                 .rvalid
		  .emif_ddr4a_mem_0_mem_cke               (DDR4A_CKE),               //  output,    width = 1,               emif_ddr4a_mem_0.mem_cke
        .emif_ddr4a_mem_0_mem_odt               (DDR4A_ODT),               //  output,    width = 1,                               .mem_odt
        .emif_ddr4a_mem_0_mem_cs_n              (DDR4A_CS_n),              //  output,    width = 1,                               .mem_cs_n
        .emif_ddr4a_mem_0_mem_a                 (DDR4A_A),                 //  output,   width = 17,                               .mem_a
        .emif_ddr4a_mem_0_mem_ba                (DDR4A_BA),                //  output,    width = 2,                               .mem_ba
        .emif_ddr4a_mem_0_mem_bg                (DDR4A_BG),                //  output,    width = 1,                               .mem_bg
        .emif_ddr4a_mem_0_mem_act_n             (DDR4A_ACT_n),             //  output,    width = 1,                               .mem_act_n
        .emif_ddr4a_mem_0_mem_par               (DDR4A_PAR),               //  output,    width = 1,                               .mem_par
        .emif_ddr4a_mem_0_mem_dq                (DDR4A_DQ),                //   inout,   width = 32,                               .mem_dq
        .emif_ddr4a_mem_0_mem_dqs_t             (DDR4A_DQS),             //   inout,    width = 4,                               .mem_dqs_t
        .emif_ddr4a_mem_0_mem_dqs_c             (DDR4A_DQS_n),             //   inout,    width = 4,                               .mem_dqs_c
        .emif_ddr4a_mem_0_mem_alert_n           (DDR4A_ALERT_n),           //   input,    width = 1,                               .mem_alert_n
        .emif_ddr4a_mem_0_mem_dbi_n             (DDR4A_DBI_n),             //   inout,    width = 4,                               .mem_dbi_n
        .emif_ddr4a_mem_ck_0_mem_ck_t           (DDR4A_CK),           //  output,    width = 1,            emif_ddr4a_mem_ck_0.mem_ck_t
        .emif_ddr4a_mem_ck_0_mem_ck_c           (DDR4A_CK_n),           //  output,    width = 1,                               .mem_ck_c
        .emif_ddr4a_mem_reset_n_mem_reset_n     (DDR4A_RESET_n),     //  output,    width = 1,         emif_ddr4a_mem_reset_n.mem_reset_n
        .emif_ddr4a_oct_0_oct_rzqin             (DDR4A_RZQ),             //   input,    width = 1,               emif_ddr4a_oct_0.oct_rzqin
        .emif_ddr4a_ref_clk_clk                 (DDR4A_REFCLK_p),                 //   input,    width = 1,             emif_ddr4a_ref_clk.clk
		  .emif_ddr4b_s0_axi4_ctrl_ready_reset_n  (emif_rst_n_ddr4b),  //  output,    width = 1,  emif_ddr4b_s0_axi4_ctrl_ready.reset_n
        .emif_ddr4b_s0_axi4_awaddr              (DDR4B_awaddr),              //   input,   width = 32,             emif_ddr4b_s0_axi4.awaddr
        .emif_ddr4b_s0_axi4_awburst             (DDR4B_awburst),             //   input,    width = 2,                                 .awburst
        .emif_ddr4b_s0_axi4_awid                (DDR4B_awid),                //   input,    width = 7,                                 .awid
        .emif_ddr4b_s0_axi4_awlen               (DDR4B_awlen),               //   input,    width = 8,                                 .awlen
        .emif_ddr4b_s0_axi4_awlock              (DDR4B_awlock),              //   input,    width = 1,                                 .awlock
        .emif_ddr4b_s0_axi4_awqos               (0),               //   input,    width = 4,                                 .awqos
        .emif_ddr4b_s0_axi4_awsize              (DDR4B_awsize),              //   input,    width = 3,                                 .awsize
        .emif_ddr4b_s0_axi4_awvalid             (DDR4B_awvalid),             //   input,    width = 1,                                 .awvalid
        .emif_ddr4b_s0_axi4_awuser              (0),              //   input,   width = 14,                                 .awuser
        .emif_ddr4b_s0_axi4_awprot              (DDR4B_awprot),              //   input,    width = 3,                                 .awprot
        .emif_ddr4b_s0_axi4_awready             (DDR4B_awready),             //  output,    width = 1,                                 .awready
        .emif_ddr4b_s0_axi4_araddr              (DDR4B_araddr),              //   input,   width = 32,                                 .araddr
        .emif_ddr4b_s0_axi4_arburst             (DDR4B_arburst),             //   input,    width = 2,                                 .arburst
        .emif_ddr4b_s0_axi4_arid                (DDR4B_arid),                //   input,    width = 7,                                 .arid
        .emif_ddr4b_s0_axi4_arlen               (DDR4B_arlen),               //   input,    width = 8,                                 .arlen
        .emif_ddr4b_s0_axi4_arlock              (DDR4B_arlock),              //   input,    width = 1,                                 .arlock
        .emif_ddr4b_s0_axi4_arqos               (0),               //   input,    width = 4,                                 .arqos
        .emif_ddr4b_s0_axi4_arsize              (DDR4B_arsize),              //   input,    width = 3,                                 .arsize
        .emif_ddr4b_s0_axi4_arvalid             (DDR4B_arvalid),             //   input,    width = 1,                                 .arvalid
        .emif_ddr4b_s0_axi4_aruser              (0),              //   input,   width = 14,                                 .aruser
        .emif_ddr4b_s0_axi4_arprot              (DDR4B_arprot),              //   input,    width = 3,                                 .arprot
        .emif_ddr4b_s0_axi4_arready             (DDR4B_arready),             //  output,    width = 1,                                 .arready
        .emif_ddr4b_s0_axi4_wdata               (DDR4B_wdata),               //   input,  width = 256,                                 .wdata
        .emif_ddr4b_s0_axi4_wstrb               (DDR4B_wstrb),               //   input,   width = 32,                                 .wstrb
        .emif_ddr4b_s0_axi4_wlast               (DDR4B_wlast),               //   input,    width = 1,                                 .wlast
        .emif_ddr4b_s0_axi4_wvalid              (DDR4B_wvalid),              //   input,    width = 1,                                 .wvalid
        .emif_ddr4b_s0_axi4_wuser               (0),               //   input,   width = 32,                                 .wuser
        .emif_ddr4b_s0_axi4_wready              (DDR4B_wready),              //  output,    width = 1,                                 .wready
        .emif_ddr4b_s0_axi4_bready              (DDR4B_bready),              //   input,    width = 1,                                 .bready
        .emif_ddr4b_s0_axi4_bid                 (DDR4B_bid),                 //  output,    width = 7,                                 .bid
        .emif_ddr4b_s0_axi4_bresp               (DDR4B_bresp),               //  output,    width = 2,                                 .bresp
        .emif_ddr4b_s0_axi4_bvalid              (DDR4B_bvalid),              //  output,    width = 1,                                 .bvalid
        .emif_ddr4b_s0_axi4_rready              (DDR4B_rready),              //   input,    width = 1,                                 .rready
        .emif_ddr4b_s0_axi4_ruser               (),              //  output,   width = 32,                                 .ruser
        .emif_ddr4b_s0_axi4_rdata               (DDR4B_rdata),               //  output,  width = 256,                                 .rdata
        .emif_ddr4b_s0_axi4_rid                 (DDR4B_rid),                 //  output,    width = 7,                                 .rid
        .emif_ddr4b_s0_axi4_rlast               (DDR4B_rlast),               //  output,    width = 1,                                 .rlast
        .emif_ddr4b_s0_axi4_rresp               (DDR4B_rresp),               //  output,    width = 2,                                 .rresp
        .emif_ddr4b_s0_axi4_rvalid              (DDR4B_rvalid),              //  output,    width = 1,                                 .rvalid
        .emif_ddr4b_s0_axi4lite_clock_clk       (AXI_LITE_CLK),       //   input,    width = 1,   emif_ddr4b_s0_axi4lite_clock.clk
        .emif_ddr4b_s0_axi4lite_reset_n_reset_n (reset_handle), //   input,    width = 1, emif_ddr4b_s0_axi4lite_reset_n.reset_n
        .emif_ddr4b_s0_axi4lite_awaddr          (DDR4B_axil_driver_awaddr),          //   input,   width = 27,         emif_ddr4b_s0_axi4lite.awaddr
        .emif_ddr4b_s0_axi4lite_awprot          (DDR4B_axil_driver_awprot),          //   input,    width = 3,                                 .awprot
        .emif_ddr4b_s0_axi4lite_awvalid         (DDR4B_axil_driver_awvalid),         //   input,    width = 1,                                 .awvalid
        .emif_ddr4b_s0_axi4lite_awready         (DDR4B_axil_driver_awready),         //  output,    width = 1,                                 .awready
        .emif_ddr4b_s0_axi4lite_araddr          (DDR4B_axil_driver_araddr),          //   input,   width = 27,                                 .araddr
        .emif_ddr4b_s0_axi4lite_arprot          (DDR4B_axil_driver_arprot),          //   input,    width = 3,                                 .arprot
        .emif_ddr4b_s0_axi4lite_arvalid         (DDR4B_axil_driver_arvalid),         //   input,    width = 1,                                 .arvalid
        .emif_ddr4b_s0_axi4lite_arready         (DDR4B_axil_driver_arready),         //  output,    width = 1,                                 .arready
        .emif_ddr4b_s0_axi4lite_wdata           (DDR4B_axil_driver_wdata),           //   input,   width = 32,                                 .wdata
        .emif_ddr4b_s0_axi4lite_wstrb           (DDR4B_axil_driver_wstrb),           //   input,    width = 4,                                 .wstrb
        .emif_ddr4b_s0_axi4lite_wvalid          (DDR4B_axil_driver_wvalid),          //   input,    width = 1,                                 .wvalid
        .emif_ddr4b_s0_axi4lite_wready          (DDR4B_axil_driver_wready),          //  output,    width = 1,                                 .wready
        .emif_ddr4b_s0_axi4lite_bready          (DDR4B_axil_driver_bready),          //   input,    width = 1,                                 .bready
        .emif_ddr4b_s0_axi4lite_bresp           (DDR4B_axil_driver_bresp),           //  output,    width = 2,                                 .bresp
        .emif_ddr4b_s0_axi4lite_bvalid          (DDR4B_axil_driver_bvalid),          //  output,    width = 1,                                 .bvalid
        .emif_ddr4b_s0_axi4lite_rready          (DDR4B_axil_driver_rready),          //   input,    width = 1,                                 .rready
        .emif_ddr4b_s0_axi4lite_rdata           (DDR4B_axil_driver_rdata),           //  output,   width = 32,                                 .rdata
        .emif_ddr4b_s0_axi4lite_rresp           (DDR4B_axil_driver_rresp),           //  output,    width = 2,                                 .rresp
        .emif_ddr4b_s0_axi4lite_rvalid          (DDR4B_axil_driver_rvalid),          //  output,    width = 1,                                 .rvalid
		  .emif_ddr4b_mem_0_mem_cke               (DDR4B_CKE),               //  output,    width = 1,               emif_ddr4b_mem_0.mem_cke
        .emif_ddr4b_mem_0_mem_odt               (DDR4B_ODT),               //  output,    width = 1,                               .mem_odt
        .emif_ddr4b_mem_0_mem_cs_n              (DDR4B_CS_n),              //  output,    width = 1,                               .mem_cs_n
        .emif_ddr4b_mem_0_mem_a                 (DDR4B_A),                 //  output,   width = 17,                               .mem_a
        .emif_ddr4b_mem_0_mem_ba                (DDR4B_BA),                //  output,    width = 2,                               .mem_ba
        .emif_ddr4b_mem_0_mem_bg                (DDR4B_BG),                //  output,    width = 1,                               .mem_bg
        .emif_ddr4b_mem_0_mem_act_n             (DDR4B_ACT_n),             //  output,    width = 1,                               .mem_act_n
        .emif_ddr4b_mem_0_mem_par               (DDR4B_PAR),               //  output,    width = 1,                               .mem_par
        .emif_ddr4b_mem_0_mem_dq                (DDR4B_DQ),                //   inout,   width = 32,                               .mem_dq
        .emif_ddr4b_mem_0_mem_dqs_t             (DDR4B_DQS),             //   inout,    width = 4,                               .mem_dqs_t
        .emif_ddr4b_mem_0_mem_dqs_c             (DDR4B_DQS_n),             //   inout,    width = 4,                               .mem_dqs_c
        .emif_ddr4b_mem_0_mem_alert_n           (DDR4B_ALERT_n),           //   input,    width = 1,                               .mem_alert_n
        .emif_ddr4b_mem_0_mem_dbi_n             (DDR4B_DBI_n),             //   inout,    width = 4,                               .mem_dbi_n
        .emif_ddr4b_mem_ck_0_mem_ck_t           (DDR4B_CK),           //  output,    width = 1,            emif_ddr4b_mem_ck_0.mem_ck_t
        .emif_ddr4b_mem_ck_0_mem_ck_c           (DDR4B_CK_n),           //  output,    width = 1,                               .mem_ck_c
        .emif_ddr4b_mem_reset_n_mem_reset_n     (DDR4B_RESET_n),     //  output,    width = 1,         emif_ddr4b_mem_reset_n.mem_reset_n
        .emif_ddr4b_oct_0_oct_rzqin             (DDR4B_RZQ),             //   input,    width = 1,               emif_ddr4b_oct_0.oct_rzqin
        .emif_ddr4b_ref_clk_clk                 (DDR4B_REFCLK_p),                 //   input,    width = 1,             emif_ddr4b_ref_clk.clk
		  .iopll_refclk_clk                       (CLK_100_B2B_p),                       //   input,    width = 1,                   iopll_refclk.clk
        .iopll_locked_export                    (DDR4_IOPLL_Lock),                    //  output,    width = 1,                   iopll_locked.export
        .iopll_reset_reset                      (ninit_done),                      //   input,    width = 1,                    iopll_reset.reset
        .iopll_outclk_axi4_lite_clk             (AXI_LITE_CLK),             //  output,    width = 1,         iopll_outclk_axi4_lite.clk
        .reset_bridge_in_reset_reset_n          (reset_handle)             //   input,    width = 1,          reset_bridge_in_reset.reset
    );

localparam PIPEDEPTH = 15;
hyper_pipe #(.WIDTH(1), .NUM_PIPES(PIPEDEPTH)) read_data_pipe
			(.din(pipe_aresetn), .dout (aresetn), .clk (aclk));
			
			
	 

/////////////////// DDR4(A) Test ///////////////////	 

//DDR4A Calibration (AXI-Lite)
wire  												DDR4A_axil_driver_clk;
wire  												DDR4A_axil_driver_rst_n;
wire  [AXIL_DRIVER_ADDRESS_WIDTH - 1:0]	DDR4A_axil_driver_araddr;
wire  [2:0]											DDR4A_axil_driver_arprot;
wire  												DDR4A_axil_driver_arvalid;
wire  												DDR4A_axil_driver_arready;
wire  [31:0]										DDR4A_axil_driver_rdata;
wire  [1:0]											DDR4A_axil_driver_rresp;
wire  												DDR4A_axil_driver_rvalid;
wire                                      DDR4A_axil_driver_rready;
wire   [AXIL_DRIVER_ADDRESS_WIDTH - 1:0]  DDR4A_axil_driver_awaddr;
wire   [2:0]                              DDR4A_axil_driver_awprot;
wire                                      DDR4A_axil_driver_awvalid;
wire                                      DDR4A_axil_driver_awready;
wire   [31:0]                             DDR4A_axil_driver_wdata;
wire   [3:0]                              DDR4A_axil_driver_wstrb;
wire                                      DDR4A_axil_driver_wvalid;
wire                                      DDR4A_axil_driver_wready;
wire   [1:0]                              DDR4A_axil_driver_bresp;
wire                                      DDR4A_axil_driver_bvalid;
wire                                      DDR4A_axil_driver_bready;
wire													DDR4A_cal_done_rst_n;
 
axil_driver_calibration DDR4A_axil_driver_calibration_Inst(

   /*input    wire                                         */    .axil_driver_clk(AXI_LITE_CLK),
   /*input    wire                                         */    .axil_driver_rst_n(DDR4_IOPLL_Lock),
   /*output   wire     [AXIL_DRIVER_ADDRESS_WIDTH - 1:0]   */    .axil_driver_araddr(DDR4A_axil_driver_araddr),
   /*output   wire     [2:0]                               */    .axil_driver_arprot(DDR4A_axil_driver_arprot),
   /*output   wire                                         */    .axil_driver_arvalid(DDR4A_axil_driver_arvalid),
   /*input    wire                                         */    .axil_driver_arready(DDR4A_axil_driver_arready),
   /*input    wire     [31:0]                              */    .axil_driver_rdata(DDR4A_axil_driver_rdata),
   /*input    wire     [1:0]                               */    .axil_driver_rresp(DDR4A_axil_driver_rresp),
   /*input    wire                                         */    .axil_driver_rvalid(DDR4A_axil_driver_rvalid),
	
   /*output   wire                                         */    .axil_driver_rready(DDR4A_axil_driver_rready),
   /*output   wire     [AXIL_DRIVER_ADDRESS_WIDTH - 1:0]   */    .axil_driver_awaddr(DDR4A_axil_driver_awaddr),
   /*output   wire     [2:0]                               */    .axil_driver_awprot(DDR4A_axil_driver_awprot),
   /*output   wire                                         */    .axil_driver_awvalid(DDR4A_axil_driver_awvalid),
   /*input    wire                                         */    .axil_driver_awready(DDR4A_axil_driver_awready),
   /*output   wire     [31:0]                              */    .axil_driver_wdata(DDR4A_axil_driver_wdata),
   /*output   wire     [3:0]                               */    .axil_driver_wstrb(DDR4A_axil_driver_wstrb),
   /*output   wire                                         */    .axil_driver_wvalid(DDR4A_axil_driver_wvalid),
   /*input    wire                                         */    .axil_driver_wready(DDR4A_axil_driver_wready),
   /*input    wire     [1:0]                               */    .axil_driver_bresp(DDR4A_axil_driver_bresp),
   /*input    wire                                         */    .axil_driver_bvalid(DDR4A_axil_driver_bvalid),
   /*output   wire                                         */    .axil_driver_bready(DDR4A_axil_driver_bready),
   /*output   wire                                         */    .cal_done_rst_n(DDR4A_cal_done_rst_n)
	
);
defparam DDR4A_axil_driver_calibration_Inst.AXIL_DRIVER_ADDRESS_WIDTH = AXIL_DRIVER_ADDRESS_WIDTH; 
  
  
// DDR4A RW Test 
parameter DDR4A_AXI_ADDR_W 			= 32;
parameter DDR4A_AXI_ADDR_W_COUNT	= 32'h2000;
parameter DDR4A_AXI_ADDR_W_START	= 32'h00_0000;
parameter DDR4A_AXI_ADDR_W_END 	= 32'hffff_e000;
parameter DDR4A_AXI_DATA_W 			= 256;
parameter DDR4A_AXI_BURST_LENGHT 	= 8'hff;
parameter DDR4A_AXI_ID_W			 	= 4;

wire 											DDR4A_awvalid;	
wire 											DDR4A_awready;  
wire	[3:0]									DDR4A_awid;  
wire 	[DDR4A_AXI_ADDR_W-1:0]  		DDR4A_awaddr;
wire 	[7:0]        						DDR4A_awlen;
wire 	[2:0]        						DDR4A_awsize;
wire 	[1:0]        						DDR4A_awburst;
wire             							DDR4A_awlock;
wire 	[3:0]        						DDR4A_awcache;
wire 	[2:0]        						DDR4A_awprot;
wire            							DDR4A_wvalid;
wire             							DDR4A_wready;
wire 	[DDR4A_AXI_DATA_W-1:0] 			DDR4A_wdata;
wire 	[(DDR4A_AXI_DATA_W>>3)-1:0] 	DDR4A_wstrb;
wire             							DDR4A_wlast;
  wire             						DDR4A_bvalid;
  wire            						DDR4A_bready;
  wire 	[3:0]  							DDR4A_bid;
  wire 	[1:0]      						DDR4A_bresp;
  wire            						DDR4A_arvalid;
  wire             						DDR4A_arready;
  wire 	[3:0]  							DDR4A_arid;
  wire 	[DDR4A_AXI_ADDR_W-1:0]  	DDR4A_araddr;
  wire 	[7:0]        					DDR4A_arlen;
  wire 	[2:0]        					DDR4A_arsize;
  wire 	[1:0]        					DDR4A_arburst;
  wire             						DDR4A_arlock;
  wire 	[3:0]        					DDR4A_arcache;
  wire 	[2:0]        					DDR4A_arprot;
  wire             						DDR4A_rvalid;
  wire            						DDR4A_rready;
  wire 	[3:0]  							DDR4A_rid;
  wire 	[DDR4A_AXI_DATA_W-1:0]  	DDR4A_rdata;
  wire 	[1:0]        					DDR4A_rresp;
  wire             						DDR4A_rlast;
//  wire           							DDR4A_test_pass;
  


 Axi4_master_RW DDR4A_axi4_master_Inst(
  /* input */             				.aclk(aclk),
  /* input */             				.aresetn(aresetn),
  /* input */             				.insert_error_n(BUTTON[0]),

  // AXI4 Master 
  /* output */            				.awvalid(DDR4A_awvalid),
  /* input */             				.awready(DDR4A_awready),
  /* output [ID_WIDTH-1:0]*/    		.awid(DDR4A_awid),
  /* output [ADDR_WIDTH-1:0]*/  		.awaddr(DDR4A_awaddr),
  /* output [7:0]*/        			.awlen(DDR4A_awlen),
  /* output [2:0]*/        			.awsize(DDR4A_awsize),
  /* output [1:0]*/        			.awburst(DDR4A_awburst),
  /* output */             			.awlock(DDR4A_awlock),
  /* output [3:0]*/        			.awcache(DDR4A_awcache),
  /* output [2:0]*/        			.awprot(DDR4A_awprot),

  /* output */            				.wvalid(DDR4A_wvalid),
  /* input */             				.wready(DDR4A_wready),
  /* output [DATA_WIDTH-1:0]*/  		.wdata(DDR4A_wdata),
  /* output [(DATA_WIDTH/8)-1:0]*/  .wstrb(DDR4A_wstrb),
  /* output */             			.wlast(DDR4A_wlast),

  /* input */             				.bvalid(DDR4A_bvalid),
  /* output */            				.bready(DDR4A_bready),
  /* input [ID_WIDTH-1:0]*/  			.bid(DDR4A_bid),
  /* input [1:0]*/        				.bresp(DDR4A_bresp),

  /* output */            				.arvalid(DDR4A_arvalid),
  /* input */             				.arready(DDR4A_arready),
  /* output [ID_WIDTH-1:0]*/  		.arid(DDR4A_arid),
  /* output [ADDR_WIDTH-1:0]*/  		.araddr(DDR4A_araddr),
  /* output [7:0]*/        			.arlen(DDR4A_arlen),
  /* output [2:0]*/        			.arsize(DDR4A_arsize),
  /* output [1:0]*/        			.arburst(DDR4A_arburst),
  /* output */             			.arlock(DDR4A_arlock),
  /* output [3:0]*/        			.arcache(DDR4A_arcache),
  /* output [2:0]*/        			.arprot(DDR4A_arprot),

  /* input */             				.rvalid(DDR4A_rvalid),
  /* output */            				.rready(DDR4A_rready),
  /* input [ID_WIDTH-1:0]*/  			.rid(DDR4A_rid),
  /* input [DATA_WIDTH-1:0]*/  		.rdata(DDR4A_rdata),
  /* input [1:0]*/        				.rresp(DDR4A_rresp),
  /* input */             				.rlast(DDR4A_rlast),

  // 
  /* output */            				.test_pass(ddr4a_test_pass),
  /* output */            				.test_complete(ddr4a_test_complete)
  
  );
defparam DDR4A_axi4_master_Inst.DDR4_AXI_ADDR_W 			= DDR4A_AXI_ADDR_W; 
defparam DDR4A_axi4_master_Inst.DDR4_AXI_ADDR_W_COUNT 	= DDR4A_AXI_ADDR_W_COUNT; 
defparam DDR4A_axi4_master_Inst.DDR4_AXI_ADDR_W_START 	= DDR4A_AXI_ADDR_W_START; 
defparam DDR4A_axi4_master_Inst.DDR4_AXI_ADDR_W_END 		= DDR4A_AXI_ADDR_W_END; 
defparam DDR4A_axi4_master_Inst.DDR4_AXI_DATA_W 			= DDR4A_AXI_DATA_W; 
defparam DDR4A_axi4_master_Inst.DDR4_AXI_BURST_LENGHT 	= DDR4A_AXI_BURST_LENGHT; 
defparam DDR4A_axi4_master_Inst.DDR4_AXI_ID_W 				= DDR4A_AXI_ID_W; 




/////////////////// LPDDR4(B) Test ///////////////////	 

//DDR4B Calibration (AXI-Lite)
wire  												DDR4B_axil_driver_clk;
wire  												DDR4B_axil_driver_rst_n;
wire  [AXIL_DRIVER_ADDRESS_WIDTH - 1:0]	DDR4B_axil_driver_araddr;
wire  [2:0]											DDR4B_axil_driver_arprot;
wire  												DDR4B_axil_driver_arvalid;
wire  												DDR4B_axil_driver_arready;
wire  [31:0]										DDR4B_axil_driver_rdata;
wire  [1:0]											DDR4B_axil_driver_rresp;
wire  												DDR4B_axil_driver_rvalid;
wire                                      DDR4B_axil_driver_rready;
wire   [AXIL_DRIVER_ADDRESS_WIDTH - 1:0]  DDR4B_axil_driver_awaddr;
wire   [2:0]                              DDR4B_axil_driver_awprot;
wire                                      DDR4B_axil_driver_awvalid;
wire                                      DDR4B_axil_driver_awready;
wire   [31:0]                             DDR4B_axil_driver_wdata;
wire   [3:0]                              DDR4B_axil_driver_wstrb;
wire                                      DDR4B_axil_driver_wvalid;
wire                                      DDR4B_axil_driver_wready;
wire   [1:0]                              DDR4B_axil_driver_bresp;
wire                                      DDR4B_axil_driver_bvalid;
wire                                      DDR4B_axil_driver_bready;
wire													DDR4B_cal_done_rst_n;
 
axil_driver_calibration DDR4B_axil_driver_calibration_Inst(

   /*input    wire                                         */    .axil_driver_clk(AXI_LITE_CLK),
   /*input    wire                                         */    .axil_driver_rst_n(DDR4_IOPLL_Lock),
   /*output   wire     [AXIL_DRIVER_ADDRESS_WIDTH - 1:0]   */    .axil_driver_araddr(DDR4B_axil_driver_araddr),
   /*output   wire     [2:0]                               */    .axil_driver_arprot(DDR4B_axil_driver_arprot),
   /*output   wire                                         */    .axil_driver_arvalid(DDR4B_axil_driver_arvalid),
   /*input    wire                                         */    .axil_driver_arready(DDR4B_axil_driver_arready),
   /*input    wire     [31:0]                              */    .axil_driver_rdata(DDR4B_axil_driver_rdata),
   /*input    wire     [1:0]                               */    .axil_driver_rresp(DDR4B_axil_driver_rresp),
   /*input    wire                                         */    .axil_driver_rvalid(DDR4B_axil_driver_rvalid),
	
   /*output   wire                                         */    .axil_driver_rready(DDR4B_axil_driver_rready),
   /*output   wire     [AXIL_DRIVER_ADDRESS_WIDTH - 1:0]   */    .axil_driver_awaddr(DDR4B_axil_driver_awaddr),
   /*output   wire     [2:0]                               */    .axil_driver_awprot(DDR4B_axil_driver_awprot),
   /*output   wire                                         */    .axil_driver_awvalid(DDR4B_axil_driver_awvalid),
   /*input    wire                                         */    .axil_driver_awready(DDR4B_axil_driver_awready),
   /*output   wire     [31:0]                              */    .axil_driver_wdata(DDR4B_axil_driver_wdata),
   /*output   wire     [3:0]                               */    .axil_driver_wstrb(DDR4B_axil_driver_wstrb),
   /*output   wire                                         */    .axil_driver_wvalid(DDR4B_axil_driver_wvalid),
   /*input    wire                                         */    .axil_driver_wready(DDR4B_axil_driver_wready),
   /*input    wire     [1:0]                               */    .axil_driver_bresp(DDR4B_axil_driver_bresp),
   /*input    wire                                         */    .axil_driver_bvalid(DDR4B_axil_driver_bvalid),
   /*output   wire                                         */    .axil_driver_bready(DDR4B_axil_driver_bready),
   /*output   wire                                         */    .cal_done_rst_n(DDR4B_cal_done_rst_n)
	
);
defparam DDR4B_axil_driver_calibration_Inst.AXIL_DRIVER_ADDRESS_WIDTH = AXIL_DRIVER_ADDRESS_WIDTH; 
  
 
// DDR4B RW Test 
parameter DDR4B_AXI_ADDR_W 			= 32;
parameter DDR4B_AXI_ADDR_W_COUNT	= 32'h2000;
parameter DDR4B_AXI_ADDR_W_START	= 32'h00_0000;
parameter DDR4B_AXI_ADDR_W_END 	= 32'hffff_e000;
parameter DDR4B_AXI_DATA_W 			= 256;
parameter DDR4B_AXI_BURST_LENGHT 	= 8'hff;
parameter DDR4B_AXI_ID_W			 	= 4;

wire 											DDR4B_awvalid;	
wire 											DDR4B_awready;  
wire	[3:0]									DDR4B_awid;  
wire 	[DDR4B_AXI_ADDR_W-1:0]  		DDR4B_awaddr;
wire 	[7:0]        						DDR4B_awlen;
wire 	[2:0]        						DDR4B_awsize;
wire 	[1:0]        						DDR4B_awburst;
wire             							DDR4B_awlock;
wire 	[3:0]        						DDR4B_awcache;
wire 	[2:0]        						DDR4B_awprot;
wire            							DDR4B_wvalid;
wire             							DDR4B_wready;
wire 	[DDR4B_AXI_DATA_W-1:0] 		DDR4B_wdata;
wire 	[(DDR4B_AXI_DATA_W>>3)-1:0] DDR4B_wstrb;
wire             							DDR4B_wlast;
  wire             						DDR4B_bvalid;
  wire            						DDR4B_bready;
  wire 	[3:0]  							DDR4B_bid;
  wire 	[1:0]      						DDR4B_bresp;
  wire            						DDR4B_arvalid;
  wire             						DDR4B_arready;
  wire 	[3:0]  							DDR4B_arid;
  wire 	[DDR4B_AXI_ADDR_W-1:0]  	DDR4B_araddr;
  wire 	[7:0]        					DDR4B_arlen;
  wire 	[2:0]        					DDR4B_arsize;
  wire 	[1:0]        					DDR4B_arburst;
  wire             						DDR4B_arlock;
  wire 	[3:0]        					DDR4B_arcache;
  wire 	[2:0]        					DDR4B_arprot;
  wire             						DDR4B_rvalid;
  wire            						DDR4B_rready;
  wire 	[3:0]  							DDR4B_rid;
  wire 	[DDR4B_AXI_DATA_W-1:0]  	DDR4B_rdata;
  wire 	[1:0]        					DDR4B_rresp;
  wire             						DDR4B_rlast;
//  wire           							DDR4B_test_pass;
  


 Axi4_master_RW DDR4B_axi4_master_Inst(
  /* input */             				.aclk(aclk),
  /* input */             				.aresetn(aresetn),
  /* input */             				.insert_error_n(BUTTON[0]),

  // AXI4 Master 
  /* output */            				.awvalid(DDR4B_awvalid),
  /* input */             				.awready(DDR4B_awready),
  /* output [ID_WIDTH-1:0]*/    		.awid(DDR4B_awid),
  /* output [ADDR_WIDTH-1:0]*/  		.awaddr(DDR4B_awaddr),
  /* output [7:0]*/        			.awlen(DDR4B_awlen),
  /* output [2:0]*/        			.awsize(DDR4B_awsize),
  /* output [1:0]*/        			.awburst(DDR4B_awburst),
  /* output */             			.awlock(DDR4B_awlock),
  /* output [3:0]*/        			.awcache(DDR4B_awcache),
  /* output [2:0]*/        			.awprot(DDR4B_awprot),

  /* output */            				.wvalid(DDR4B_wvalid),
  /* input */             				.wready(DDR4B_wready),
  /* output [DATA_WIDTH-1:0]*/  		.wdata(DDR4B_wdata),
  /* output [(DATA_WIDTH/8)-1:0]*/  .wstrb(DDR4B_wstrb),
  /* output */             			.wlast(DDR4B_wlast),

  /* input */             				.bvalid(DDR4B_bvalid),
  /* output */            				.bready(DDR4B_bready),
  /* input [ID_WIDTH-1:0]*/  			.bid(DDR4B_bid),
  /* input [1:0]*/        				.bresp(DDR4B_bresp),

  /* output */            				.arvalid(DDR4B_arvalid),
  /* input */             				.arready(DDR4B_arready),
  /* output [ID_WIDTH-1:0]*/  		.arid(DDR4B_arid),
  /* output [ADDR_WIDTH-1:0]*/  		.araddr(DDR4B_araddr),
  /* output [7:0]*/        			.arlen(DDR4B_arlen),
  /* output [2:0]*/        			.arsize(DDR4B_arsize),
  /* output [1:0]*/        			.arburst(DDR4B_arburst),
  /* output */             			.arlock(DDR4B_arlock),
  /* output [3:0]*/        			.arcache(DDR4B_arcache),
  /* output [2:0]*/        			.arprot(DDR4B_arprot),

  /* input */             				.rvalid(DDR4B_rvalid),
  /* output */            				.rready(DDR4B_rready),
  /* input [ID_WIDTH-1:0]*/  			.rid(DDR4B_rid),
  /* input [DATA_WIDTH-1:0]*/  		.rdata(DDR4B_rdata),
  /* input [1:0]*/        				.rresp(DDR4B_rresp),
  /* input */             				.rlast(DDR4B_rlast),

  // 
  /* output */            				.test_pass(ddr4b_test_pass),
  /* output */            				.test_complete(ddr4b_test_complete)
  
  );
defparam DDR4B_axi4_master_Inst.DDR4_AXI_ADDR_W 			= DDR4B_AXI_ADDR_W; 
defparam DDR4B_axi4_master_Inst.DDR4_AXI_ADDR_W_COUNT 	= DDR4B_AXI_ADDR_W_COUNT; 
defparam DDR4B_axi4_master_Inst.DDR4_AXI_ADDR_W_START 	= DDR4B_AXI_ADDR_W_START; 
defparam DDR4B_axi4_master_Inst.DDR4_AXI_ADDR_W_END 		= DDR4B_AXI_ADDR_W_END; 
defparam DDR4B_axi4_master_Inst.DDR4_AXI_DATA_W 			= DDR4B_AXI_DATA_W; 
defparam DDR4B_axi4_master_Inst.DDR4_AXI_BURST_LENGHT 	= DDR4B_AXI_BURST_LENGHT; 
defparam DDR4B_axi4_master_Inst.DDR4_AXI_ID_W 				= DDR4B_AXI_ID_W; 

wire heart_led;
heart_beat  heart_beat_0 ( .CLK(CLK_50_B5A ),     .CLK_FREQ  (50_000_000) , . CK_1HZ (heart_led) );

assign LED[0] = DDR4A_cal_done_rst_n ? ~(ddr4a_test_pass & ddr4a_test_complete) : heart_led;
assign LED[1] = DDR4B_cal_done_rst_n ? ~(ddr4b_test_pass & ddr4b_test_complete) : heart_led;
assign LED[2] = heart_led;
assign LED[3] = heart_led;


endmodule
