# (C) 2001-2024 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


## SDC ##

set dir_name [file dirname [info script]] 
#source "${dir_name}/xcvr_usb3_x2_refclk_intel_directphy_gts_0_intel_directphy_gts_500_j6hrspi_sip_ip_parameters.tcl"
source "${dir_name}/xcvr_sdc_utils.tcl"

set module_name xcvr_usb3_x2_refclk_intel_directphy_gts_0_intel_directphy_gts_500_j6hrspi_sip


## set global variables
global ::ip_sdc_debug


 set ip_sdc_debug 0
 
 ## get current IP instance 
 set ip_inst_name [get_current_instance]
 if {$ip_sdc_debug == 1} { 
	post_message -type info "IP SDC: $ip_inst_name"
 }

## Set parameters based on IP configuration, generated by HWTCL

set tx_user_clk1_mhz 250.0

set rx_user_clk1_mhz 250.0
set rx_ref_clk_mhz 100.000000

set tx_user_clk1_div 2

set rx_user_clk1_div 2
# -------------------------------------------------------------------------------------
# SETTING CLOCK CONSTRAINTS FOR XCVR USER CLOCK OUTPUTS
# -------------------------------------------------------------------------------------
# Typically there are 4 user output clocks from the Transceiver
# 2 for TX and 2 for RX
#
# -------------------------------------------------------------------------------------
# Prerequisites:
# -------------------------------------------------------------------------------------
# 1. Instantiate dummy flops in SIP which are clocked by the XCVR user clock outputs
# 2. Pass the frequency of the user clocks from HWTCL as per IP configuration
# 3. Pass the divider value used on the user clocks from HWTCL as per IP configuration
# 4. Entity associated with this SDC should be pointing to the SIP module in QIP
#
# -------------------------------------------------------------------------------------
# Goals of the script:
# -------------------------------------------------------------------------------------
# 1. Using the SIP dummy flop, trace the pin on the PLDIF (or HIP boundary)
#    which is driving the clock to this dummy flop
# 2. Using the pin found above, trace back into the timing model to 
#    find the source reference clock and register inside the HIP (or HSSI SS)
# 3. The output clock towards the fabric passes through a Divider in PLDIF
#    and hence this divider value set by user is also taken into account
# 4. Create the clock constraint for the original Ref clock source
# 5. Create the Generated clock constraint for the Reg clock 
# 6. Create the Generated clock constraint for the User clock output towards fabric
#
# -------------------------------------------------------------------------------------


    ## Compute the Source Ref Clock frequency after adjusting for Divider in PLDIF
    set clk_ref_freq_mhz(0) [expr $tx_user_clk1_mhz * $tx_user_clk1_div] 

    set clk_ref_freq_mhz(2) [expr $rx_user_clk1_mhz * $rx_user_clk1_div] 
    set clk_ref_freq_mhz(4) [expr $rx_ref_clk_mhz * 1]

    ## Load the Divider values into variables
    set clk_out_div(0) $tx_user_clk1_div 

    set clk_out_div(2) $rx_user_clk1_div 


                    set total_no_of_clocks 3*2*1
    
    
## Set the total number of clocks that need to be constrained
# -------------------------------------------------------------------------------------
# Channel Number 0
# -------------------------------------------------------------------------------------

    ## Set the exact name of the dummy registers instantiated in SIP for each clock
    set dummy_sip_flop_names_ch0(0) dummy_tx_user_clk1_reg[0]

    set dummy_sip_flop_names_ch0(2) dummy_rx_user_clk1_reg[0]
    set dummy_sip_flop_names_ch0(4) dummy_rx_ref_clk_reg[0]

    ## Set the names of the source reference clock inside the HIP
    set clk_ref_names_ch0(0) tx_user_clk1_ref[0]

    set clk_ref_names_ch0(2) rx_user_clk1_ref[0]
    set clk_ref_names_ch0(4) rx_ref_clk_ref[0]

    ## Set the names of the register clock inside the HIP
    set clk_reg_names_ch0(0) tx_user_clk1_reg[0]

    set clk_reg_names_ch0(2) rx_user_clk1_reg[0]
    set clk_reg_names_ch0(4) rx_ref_clk_reg[0]

    ## Set the names of the Output clocks
    ## These are the names that will appear in the "Clocks" report after clock creation
    set clk_out_names_ch0(0) o_tx_clkout[0]

    set clk_out_names_ch0(2) o_rx_clkout[0]
    set clk_out_names_ch0(4) o_refclk2core[0]

## Load the ref node which needs to be found
set ref_search_node_ch0(0) syspll_c0_ref 



set ref_search_node_ch0(2) syspll_c0_ref 

set ref_search_node_ch0(4) {refclk_ref$ }  


## Load the reg node which needs to be found
set reg_search_node_ch0(0) syspll_c0.reg



set reg_search_node_ch0(2) syspll_c0.reg

set reg_search_node_ch0(4) refclk_ref.reg  



    
    ## Create clock constraints for Ref/Reg/Output clocks
    ## tx clockout
     #Find clock pin/ref/reg nodes for each user clock
        set xcvrclk_pin_node_ch0(0) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch0(0) "pin" $ip_inst_name $ip_sdc_debug]
        set xcvrclk_ref_node_ch0(0) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch0(0) $ref_search_node_ch0(0) $ip_inst_name $ip_sdc_debug]
        set xcvrclk_reg_node_ch0(0) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch0(0) $reg_search_node_ch0(0) $ip_inst_name $ip_sdc_debug]
	
    #Check if clock with target "syspll_c0_ref" already exists; if yes, get the target node and clock name 
	set clk_exists "0"
	set clk_exists_node ""
	set clk_exists_name ""
	##if {$ref_search_node_ch0(0) == "syspll_c0_ref"} {
		set clk_exists [lookup_clock_target $xcvrclk_ref_node_ch0(0)]
		if {$clk_exists == 1} {
			set clk_exists_name [lookup_clock_target_name $xcvrclk_ref_node_ch0(0)]
			set clk_exists_node [get_node_info -name [get_clock_info -target $clk_exists_name]]
			if {$ip_sdc_debug == 1 && $clk_exists == 1} {
				post_message -type info "IP SDC: Clock already created with same target, skipping new clock creation: $clk_exists_name"
				#post_message -type info "Clock already created: NODE: $clk_exists_node"				
			}				
		}
	##}

   
    if { $clk_exists == 0 } {
        #create clock for source ref clock inside HIP
        create_clock \
                                      -name $ip_inst_name|$clk_ref_names_ch0(0) \
                                      -period "$clk_ref_freq_mhz(0) MHz"\
                                      -add   $xcvrclk_ref_node_ch0(0)
    }
    
    if { $clk_exists == 0 } {
    #create generated clock for reg clock inside HIP
        create_generated_clock \
                                      -name $ip_inst_name|$clk_reg_names_ch0(0) \
                                      -source $xcvrclk_ref_node_ch0(0) \
                                      -master_clock $ip_inst_name|$clk_ref_names_ch0(0) \
                                      -multiply_by 1 \
                                      -divide_by 1 \
                                      -duty_cycle 50 \
                                      -add $xcvrclk_reg_node_ch0(0)
    
    } else {
    #create generated clock for reg clock inside HIP
        create_generated_clock \
                                      -name $ip_inst_name|$clk_reg_names_ch0(0) \
                                      -source $clk_exists_node \
                                      -master_clock $clk_exists_name \
                                      -multiply_by 1 \
                                      -divide_by 1 \
                                      -duty_cycle 50 \
                                      -add $xcvrclk_reg_node_ch0(0)
    
    }
       
        #create generated clock for user output clock to Fabric/SIP
        create_generated_clock \
                                       -name $ip_inst_name|$clk_out_names_ch0(0) \
                                       -source $xcvrclk_reg_node_ch0(0) \
                                       -master_clock $ip_inst_name|$clk_reg_names_ch0(0) \
                                       -multiply_by 1 \
                                       -divide_by $clk_out_div(0) \
                                       -duty_cycle 50 \
                                       -add $xcvrclk_pin_node_ch0(0) 
	
    if { $clk_exists == 0 } {
	disable_min_pulse_width [get_clocks $ip_inst_name|$clk_ref_names_ch0(0)]
    }

#	disable_min_pulse_width [get_clocks $ip_inst_name|$clk_reg_names_ch0(0)]
#	disable_min_pulse_width [get_clocks $ip_inst_name|$clk_out_names_ch0(0)]




    ## rx clockout    
        #Find clock pin/ref/reg nodes for each user clock
        set xcvrclk_pin_node_ch0(2) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch0(2) "pin" $ip_inst_name $ip_sdc_debug]
        set xcvrclk_ref_node_ch0(2) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch0(2) $ref_search_node_ch0(2) $ip_inst_name $ip_sdc_debug]
        set xcvrclk_reg_node_ch0(2) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch0(2) $reg_search_node_ch0(2) $ip_inst_name $ip_sdc_debug]
	
        #Check if clock with target "syspll_c0_ref" already exists; if yes, get the target node and clock name 
	set clk_exists "0"
	set clk_exists_node ""
	set clk_exists_name ""
	##if {$ref_search_node_ch0(2) == "syspll_c0_ref"} {
		set clk_exists [lookup_clock_target $xcvrclk_ref_node_ch0(2)]
		if {$clk_exists == 1} {
			set clk_exists_name [lookup_clock_target_name $xcvrclk_ref_node_ch0(2)]
			set clk_exists_node [get_node_info -name [get_clock_info -target $clk_exists_name]]
			if {$ip_sdc_debug == 1 && $clk_exists == 1} {
				post_message -type info "IP SDC: Clock already created with same target, skipping new clock creation: $clk_exists_name"
				#post_message -type info "Clock already created: NODE: $clk_exists_node"				
			}				
		}
	##}
    
    if { $clk_exists == 0 } {
        #create clock for source ref clock inside HIP
        create_clock \
                                      -name $ip_inst_name|$clk_ref_names_ch0(2) \
                                      -period "$clk_ref_freq_mhz(2) MHz"\
                                      -add   $xcvrclk_ref_node_ch0(2)
    }
    
    if { $clk_exists == 0 } {
        #create generated clock for reg clock inside HIP
        create_generated_clock \
                                      -name $ip_inst_name|$clk_reg_names_ch0(2) \
                                      -source $xcvrclk_ref_node_ch0(2) \
                                      -master_clock $ip_inst_name|$clk_ref_names_ch0(2) \
                                      -multiply_by 1 \
                                      -divide_by 1 \
                                      -duty_cycle 50 \
                                      -add $xcvrclk_reg_node_ch0(2)
    } else { 
            #create generated clock for reg clock inside HIP
        create_generated_clock \
                                      -name $ip_inst_name|$clk_reg_names_ch0(2) \
                                      -source $clk_exists_node \
                                      -master_clock $clk_exists_name \
                                      -multiply_by 1 \
                                      -divide_by 1 \
                                      -duty_cycle 50 \
                                      -add $xcvrclk_reg_node_ch0(2)
    }    
    
        #create generated clock for user output clock to Fabric/SIP
        create_generated_clock \
                                       -name $ip_inst_name|$clk_out_names_ch0(2) \
                                       -source $xcvrclk_reg_node_ch0(2) \
                                       -master_clock $ip_inst_name|$clk_reg_names_ch0(2) \
                                       -multiply_by 1 \
                                       -divide_by $clk_out_div(2) \
                                       -duty_cycle 50 \
                                       -add $xcvrclk_pin_node_ch0(2) 
	
	
    if { $clk_exists == 0 } {
    disable_min_pulse_width [get_clocks $ip_inst_name|$clk_ref_names_ch0(2)]
    }
    
#    disable_min_pulse_width [get_clocks $ip_inst_name|$clk_reg_names_ch0(2)]
#    disable_min_pulse_width [get_clocks $ip_inst_name|$clk_out_names_ch0(2)]


 
        
   
    
# -------------------------------------------------------------------------------------
# Channel Number 0 end
# -------------------------------------------------------------------------------------
    
# -------------------------------------------------------------------------------------
# Channel Number 1
# -------------------------------------------------------------------------------------

    ## Set the exact name of the dummy registers instantiated in SIP for each clock
    set dummy_sip_flop_names_ch1(0) dummy_tx_user_clk1_reg[1]

    set dummy_sip_flop_names_ch1(2) dummy_rx_user_clk1_reg[1]
    set dummy_sip_flop_names_ch1(4) dummy_rx_ref_clk_reg[1]

    ## Set the names of the source reference clock inside the HIP
    set clk_ref_names_ch1(0) tx_user_clk1_ref[1]

    set clk_ref_names_ch1(2) rx_user_clk1_ref[1]
    set clk_ref_names_ch1(4) rx_ref_clk_ref[1]

    ## Set the names of the register clock inside the HIP
    set clk_reg_names_ch1(0) tx_user_clk1_reg[1]

    set clk_reg_names_ch1(2) rx_user_clk1_reg[1]
    set clk_reg_names_ch1(4) rx_ref_clk_reg[1]

    ## Set the names of the Output clocks
    ## These are the names that will appear in the "Clocks" report after clock creation
    set clk_out_names_ch1(0) o_tx_clkout[1]

    set clk_out_names_ch1(2) o_rx_clkout[1]
    set clk_out_names_ch1(4) o_refclk2core[1]

## Load the ref node which needs to be found
set ref_search_node_ch1(0) syspll_c0_ref 



set ref_search_node_ch1(2) syspll_c0_ref 

set ref_search_node_ch1(4) {refclk_ref$ }  


## Load the reg node which needs to be found
set reg_search_node_ch1(0) syspll_c0.reg



set reg_search_node_ch1(2) syspll_c0.reg

set reg_search_node_ch1(4) refclk_ref.reg  



    
    ## Create clock constraints for Ref/Reg/Output clocks
    ## tx clockout
     #Find clock pin/ref/reg nodes for each user clock
        set xcvrclk_pin_node_ch1(0) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch1(0) "pin" $ip_inst_name $ip_sdc_debug]
        set xcvrclk_ref_node_ch1(0) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch1(0) $ref_search_node_ch1(0) $ip_inst_name $ip_sdc_debug]
        set xcvrclk_reg_node_ch1(0) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch1(0) $reg_search_node_ch1(0) $ip_inst_name $ip_sdc_debug]
	
    #Check if clock with target "syspll_c0_ref" already exists; if yes, get the target node and clock name 
	set clk_exists "0"
	set clk_exists_node ""
	set clk_exists_name ""
	##if {$ref_search_node_ch1(0) == "syspll_c0_ref"} {
		set clk_exists [lookup_clock_target $xcvrclk_ref_node_ch1(0)]
		if {$clk_exists == 1} {
			set clk_exists_name [lookup_clock_target_name $xcvrclk_ref_node_ch1(0)]
			set clk_exists_node [get_node_info -name [get_clock_info -target $clk_exists_name]]
			if {$ip_sdc_debug == 1 && $clk_exists == 1} {
				post_message -type info "IP SDC: Clock already created with same target, skipping new clock creation: $clk_exists_name"
				#post_message -type info "Clock already created: NODE: $clk_exists_node"				
			}				
		}
	##}

   
    if { $clk_exists == 0 } {
        #create clock for source ref clock inside HIP
        create_clock \
                                      -name $ip_inst_name|$clk_ref_names_ch1(0) \
                                      -period "$clk_ref_freq_mhz(0) MHz"\
                                      -add   $xcvrclk_ref_node_ch1(0)
    }
    
    if { $clk_exists == 0 } {
    #create generated clock for reg clock inside HIP
        create_generated_clock \
                                      -name $ip_inst_name|$clk_reg_names_ch1(0) \
                                      -source $xcvrclk_ref_node_ch1(0) \
                                      -master_clock $ip_inst_name|$clk_ref_names_ch1(0) \
                                      -multiply_by 1 \
                                      -divide_by 1 \
                                      -duty_cycle 50 \
                                      -add $xcvrclk_reg_node_ch1(0)
    
    } else {
    #create generated clock for reg clock inside HIP
        create_generated_clock \
                                      -name $ip_inst_name|$clk_reg_names_ch1(0) \
                                      -source $clk_exists_node \
                                      -master_clock $clk_exists_name \
                                      -multiply_by 1 \
                                      -divide_by 1 \
                                      -duty_cycle 50 \
                                      -add $xcvrclk_reg_node_ch1(0)
    
    }
       
        #create generated clock for user output clock to Fabric/SIP
        create_generated_clock \
                                       -name $ip_inst_name|$clk_out_names_ch1(0) \
                                       -source $xcvrclk_reg_node_ch1(0) \
                                       -master_clock $ip_inst_name|$clk_reg_names_ch1(0) \
                                       -multiply_by 1 \
                                       -divide_by $clk_out_div(0) \
                                       -duty_cycle 50 \
                                       -add $xcvrclk_pin_node_ch1(0) 
	
    if { $clk_exists == 0 } {
	disable_min_pulse_width [get_clocks $ip_inst_name|$clk_ref_names_ch1(0)]
    }

#	disable_min_pulse_width [get_clocks $ip_inst_name|$clk_reg_names_ch1(0)]
#	disable_min_pulse_width [get_clocks $ip_inst_name|$clk_out_names_ch1(0)]




    ## rx clockout    
        #Find clock pin/ref/reg nodes for each user clock
        set xcvrclk_pin_node_ch1(2) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch1(2) "pin" $ip_inst_name $ip_sdc_debug]
        set xcvrclk_ref_node_ch1(2) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch1(2) $ref_search_node_ch1(2) $ip_inst_name $ip_sdc_debug]
        set xcvrclk_reg_node_ch1(2) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch1(2) $reg_search_node_ch1(2) $ip_inst_name $ip_sdc_debug]
	
        #Check if clock with target "syspll_c0_ref" already exists; if yes, get the target node and clock name 
	set clk_exists "0"
	set clk_exists_node ""
	set clk_exists_name ""
	##if {$ref_search_node_ch1(2) == "syspll_c0_ref"} {
		set clk_exists [lookup_clock_target $xcvrclk_ref_node_ch1(2)]
		if {$clk_exists == 1} {
			set clk_exists_name [lookup_clock_target_name $xcvrclk_ref_node_ch1(2)]
			set clk_exists_node [get_node_info -name [get_clock_info -target $clk_exists_name]]
			if {$ip_sdc_debug == 1 && $clk_exists == 1} {
				post_message -type info "IP SDC: Clock already created with same target, skipping new clock creation: $clk_exists_name"
				#post_message -type info "Clock already created: NODE: $clk_exists_node"				
			}				
		}
	##}
    
    if { $clk_exists == 0 } {
        #create clock for source ref clock inside HIP
        create_clock \
                                      -name $ip_inst_name|$clk_ref_names_ch1(2) \
                                      -period "$clk_ref_freq_mhz(2) MHz"\
                                      -add   $xcvrclk_ref_node_ch1(2)
    }
    
    if { $clk_exists == 0 } {
        #create generated clock for reg clock inside HIP
        create_generated_clock \
                                      -name $ip_inst_name|$clk_reg_names_ch1(2) \
                                      -source $xcvrclk_ref_node_ch1(2) \
                                      -master_clock $ip_inst_name|$clk_ref_names_ch1(2) \
                                      -multiply_by 1 \
                                      -divide_by 1 \
                                      -duty_cycle 50 \
                                      -add $xcvrclk_reg_node_ch1(2)
    } else { 
            #create generated clock for reg clock inside HIP
        create_generated_clock \
                                      -name $ip_inst_name|$clk_reg_names_ch1(2) \
                                      -source $clk_exists_node \
                                      -master_clock $clk_exists_name \
                                      -multiply_by 1 \
                                      -divide_by 1 \
                                      -duty_cycle 50 \
                                      -add $xcvrclk_reg_node_ch1(2)
    }    
    
        #create generated clock for user output clock to Fabric/SIP
        create_generated_clock \
                                       -name $ip_inst_name|$clk_out_names_ch1(2) \
                                       -source $xcvrclk_reg_node_ch1(2) \
                                       -master_clock $ip_inst_name|$clk_reg_names_ch1(2) \
                                       -multiply_by 1 \
                                       -divide_by $clk_out_div(2) \
                                       -duty_cycle 50 \
                                       -add $xcvrclk_pin_node_ch1(2) 
	
	
    if { $clk_exists == 0 } {
    disable_min_pulse_width [get_clocks $ip_inst_name|$clk_ref_names_ch1(2)]
    }
    
#    disable_min_pulse_width [get_clocks $ip_inst_name|$clk_reg_names_ch1(2)]
#    disable_min_pulse_width [get_clocks $ip_inst_name|$clk_out_names_ch1(2)]


 
        
   
    
# -------------------------------------------------------------------------------------
# Channel Number 1 end
# -------------------------------------------------------------------------------------
    

# -------------------------------------------------------------------------------------
# refclock2core
# -------------------------------------------------------------------------------------
  
        #Find clock pin/ref/reg nodes for each user clock
        set xcvrclk_pin_node_ch0(4) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch0(4) "pin" $ip_inst_name $ip_sdc_debug]
        set xcvrclk_ref_node_ch0(4) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch0(4) $ref_search_node_ch0(4) $ip_inst_name $ip_sdc_debug]
        set xcvrclk_reg_node_ch0(4) [xcvrphy_traverse_fanins $ip_inst_name $dummy_sip_flop_names_ch0(4) $reg_search_node_ch0(4) $ip_inst_name $ip_sdc_debug]
        
         #Check if clock with target "syspll_c0_ref" already exists; if yes, get the target node and clock name 
        set clk_exists "0"
        set clk_exists_node ""
        set clk_exists_name ""
        ##if {$ref_search_node_ch0(4) == "syspll_c0_ref"} {
            set clk_exists [lookup_clock_target $xcvrclk_ref_node_ch0(4)]
            if {$clk_exists == 1} {
                set clk_exists_name [lookup_clock_target_name $xcvrclk_ref_node_ch0(4)]
                set clk_exists_node [get_node_info -name [get_clock_info -target $clk_exists_name]]
                if {$ip_sdc_debug == 1 && $clk_exists == 1} {
                    post_message -type info "IP SDC: Clock already created with same target, skipping new clock creation: $clk_exists_name"
                    #post_message -type info "Clock already created: NODE: $clk_exists_node"				
                }				
            }
        ##}
        
        if { $clk_exists == 0 } {
            #create clock for source ref clock inside HIP
            create_clock \
                                        -name $ip_inst_name|$clk_ref_names_ch0(4) \
                                        -period "$clk_ref_freq_mhz(4) MHz"\
                                        -add   $xcvrclk_ref_node_ch0(4)
        }
        
        if { $clk_exists == 0 } {
            #create generated clock for reg clock inside HIP
            create_generated_clock \
                                        -name $ip_inst_name|$clk_reg_names_ch0(4) \
                                        -source $xcvrclk_ref_node_ch0(4) \
                                        -master_clock $ip_inst_name|$clk_ref_names_ch0(4) \
                                        -multiply_by 1 \
                                        -divide_by 1 \
                                        -duty_cycle 50 \
                                        -add $xcvrclk_reg_node_ch0(4)
        } else { 
            #create generated clock for reg clock inside HIP
            create_generated_clock \
                                        -name $ip_inst_name|$clk_reg_names_ch0(4) \
                                        -source $clk_exists_node \
                                        -master_clock $clk_exists_name \
                                        -multiply_by 1 \
                                        -divide_by 1 \
                                        -duty_cycle 50 \
                                        -add $xcvrclk_reg_node_ch0(4)
        }
            
            #create generated clock for user output clock to Fabric/SIP
            create_generated_clock \
                                        -name $ip_inst_name|$clk_out_names_ch0(4) \
                                        -source $xcvrclk_reg_node_ch0(4) \
                                        -master_clock $ip_inst_name|$clk_reg_names_ch0(4) \
                                        -multiply_by 1 \
                                        -divide_by 1 \
                                        -duty_cycle 50 \
                                        -add $xcvrclk_pin_node_ch0(4) 
        if { $clk_exists == 0 } {
        disable_min_pulse_width [get_clocks $ip_inst_name|$clk_ref_names_ch0(4)]
        }
#        disable_min_pulse_width [get_clocks $ip_inst_name|$clk_reg_names_ch0(4)]
#        disable_min_pulse_width [get_clocks $ip_inst_name|$clk_out_names_ch0(4)]
              
        
        

##-----------------------------------------------------------------------------------------------##
##----  CCG cross clock false paths                                                          ----##
##-----------------------------------------------------------------------------------------------##
for {set i 0} {$i < 1} {incr i} {
     set raddr_g_completed [get_registers -nowarn $ip_inst_name|*persystem[$i].ccg.ccg|raddr_g_completed*]
     set ccg_sr1_din_meta [get_registers -nowarn $ip_inst_name|*persystem[$i].ccg.ccg|sr1|resync_chains*.synchronizer_nocut|din_s1]
     if {[get_collection_size $raddr_g_completed] > 0 && [get_collection_size $ccg_sr1_din_meta] > 0} {
       set_max_skew -from $raddr_g_completed -to $ccg_sr1_din_meta -get_skew_value_from_clock_period src_clock_period -skew_value_multiplier 0.8
        set_net_delay -from $raddr_g_completed -to $ccg_sr1_din_meta -max -get_value_from_clock_period dst_clock_period -value_multiplier 0.8
        set_max_delay -from $raddr_g_completed -to $ccg_sr1_din_meta 100
        set_min_delay -from $raddr_g_completed -to $ccg_sr1_din_meta -100
     }
     
     
     set waddr_g_completed [get_registers -nowarn $ip_inst_name|*persystem[$i].ccg.ccg|waddr_g_completed*]
     set ccg_sr0_din_meta [get_registers -nowarn $ip_inst_name|*persystem[$i].ccg.ccg|sr0|resync_chains*.synchronizer_nocut|din_s1]
     if {[get_collection_size $waddr_g_completed] > 0 && [get_collection_size $ccg_sr0_din_meta] > 0} {
       set_max_skew -from $waddr_g_completed -to $ccg_sr0_din_meta -get_skew_value_from_clock_period src_clock_period -skew_value_multiplier 0.8
        set_net_delay -from $waddr_g_completed -to $ccg_sr0_din_meta -max -get_value_from_clock_period dst_clock_period -value_multiplier 0.8
        set_max_delay -from $waddr_g_completed -to $ccg_sr0_din_meta 100
        set_min_delay -from $waddr_g_completed -to $ccg_sr0_din_meta -100
     }
 
 
 
     set rst_sync_tx_rd_clk_dreg [get_registers -nowarn $ip_inst_name|*persystem[$i].ccg.ccg|rst_sync_tx_rd_clk|resync_chains[*].synchronizer_nocut|dreg[*]]
     set rst_sync_tx_rd_clk_dreg_clrn [get_pins -nowarn -compat $ip_inst_name|*persystem[$i].ccg.ccg|rst_sync_tx_rd_clk|resync_chains[*].synchronizer_nocut|dreg[*]|clrn]
      if {[get_collection_size $rst_sync_tx_rd_clk_dreg_clrn] > 0 && [get_collection_size $rst_sync_tx_rd_clk_dreg] > 0} {
	  set_max_delay -through $rst_sync_tx_rd_clk_dreg_clrn -to $rst_sync_tx_rd_clk_dreg 100
	  set_min_delay -through $rst_sync_tx_rd_clk_dreg_clrn -to $rst_sync_tx_rd_clk_dreg -100
      }
	  
     set rst_sync_tx_rd_clk_din [get_registers -nowarn $ip_inst_name|*persystem[$i].ccg.ccg|rst_sync_tx_rd_clk|resync_chains[*].synchronizer_nocut|din_s1]
     set rst_sync_tx_rd_clk_din_clrn [get_pins -nowarn -compat $ip_inst_name|*persystem[$i].ccg.ccg|rst_sync_tx_rd_clk|resync_chains[*].synchronizer_nocut|din_s1|clrn]
      if {[get_collection_size $rst_sync_tx_rd_clk_din_clrn] > 0 && [get_collection_size $rst_sync_tx_rd_clk_din] > 0} {
	  set_max_delay -through $rst_sync_tx_rd_clk_din_clrn -to $rst_sync_tx_rd_clk_din 100
	  set_min_delay -through $rst_sync_tx_rd_clk_din_clrn -to $rst_sync_tx_rd_clk_din -100
      }
 
	  
     set rst_sync_tx_wr_clk_dreg [get_registers -nowarn $ip_inst_name|*persystem[$i].ccg.ccg|rst_sync_tx_wr_clk|resync_chains[*].synchronizer_nocut|dreg[*]]
     set rst_sync_tx_wr_clk_dreg_clrn [get_pins -nowarn -compat $ip_inst_name|*persystem[$i].ccg.ccg|rst_sync_tx_wr_clk|resync_chains[*].synchronizer_nocut|dreg[*]|clrn]
      if {[get_collection_size $rst_sync_tx_wr_clk_dreg_clrn] > 0 && [get_collection_size $rst_sync_tx_wr_clk_dreg] > 0} {
         set_max_delay -through $rst_sync_tx_wr_clk_dreg_clrn -to $rst_sync_tx_wr_clk_dreg 100
         set_min_delay -through $rst_sync_tx_wr_clk_dreg_clrn -to $rst_sync_tx_wr_clk_dreg -100
      }
	  
     set rst_sync_tx_wr_clk_din [get_registers -nowarn $ip_inst_name|*persystem[$i].ccg.ccg|rst_sync_tx_wr_clk|resync_chains[*].synchronizer_nocut|din_s1]
     set rst_sync_tx_wr_clk_din_clrn [get_pins -nowarn -compat $ip_inst_name|*persystem[$i].ccg.ccg|rst_sync_tx_wr_clk|resync_chains[*].synchronizer_nocut|din_s1|clrn]
      if {[get_collection_size $rst_sync_tx_wr_clk_din_clrn] > 0 && [get_collection_size $rst_sync_tx_wr_clk_din] > 0} {
	 set_max_delay -through $rst_sync_tx_wr_clk_din_clrn -to $rst_sync_tx_wr_clk_din 100
	 set_min_delay -through $rst_sync_tx_wr_clk_din_clrn -to $rst_sync_tx_wr_clk_din -100
      }
}

  #--- Clock domain crossing for Div2             --- #
  #---                                            --- #
  #-------------------------------------------------- #
  ## seting group for TX  & RX source clocks
 set rx_clkout2 [get_clocks -nowarn *o_rx_clkout2[*]]
 set rx_clkout [get_clocks -nowarn *o_rx_clkout[*]]
  if {[get_collection_size $rx_clkout2] > 0 && [get_collection_size $rx_clkout] > 0} {
     set_clock_groups -asynchronous -group $rx_clkout2 -group $rx_clkout      
  }
 set tx_clkout2 [get_clocks -nowarn *o_tx_clkout2[*]]
 set tx_clkout [get_clocks -nowarn *o_tx_clkout[*]]
  if {[get_collection_size $tx_clkout2] > 0 && [get_collection_size $tx_clkout] > 0} {
    set_clock_groups -asynchronous -group $tx_clkout2 -group $tx_clkout
}



