# (C) 2001-2024 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.





# From Timequest cookbook

derive_clock_uncertainty

set CLK_STATUS      [get_clocks clk_status]
set RECONFIG_CLK    [get_clocks reconfig_clk]

set_clock_groups -asynchronous -group $RECONFIG_CLK 
set_clock_groups -asynchronous -group $CLK_STATUS 

##### False paths
set_false_path -from [get_ports {csr_rst_n}]
set_false_path -from [get_ports {reconfig_reset}]

##### From Timequest cookbook
set_clock_groups -exclusive -group [get_clocks altera_reserved_tck]
####

proc intel_eth_e40_constraint_net_delay {from_reg to_reg max_net_delay {check_exist 0} {get_pins 1} {set_skew_constraint 1} {set_mstable 1} {set_no_synchronizer 0}} {
    
    # Check for instances
    set inst [get_registers -nowarn ${to_reg}]
    
    # Check number of instances
    set inst_num [llength [query_collection -report -all $inst]]
    if {$inst_num > 0} {
        # Uncomment line below for debug purpose
        #puts "${inst_num} ${to_reg} instance(s) found"
    } else {
        # Uncomment line below for debug purpose
        #puts "No ${to_reg} instance found"
    }
 
  if {($set_skew_constraint == 0)} {   
    if {($check_exist == 0) || ($inst_num > 0)} {
        if { [string equal "quartus_sta" $::TimeQuestInfo(nameofexecutable)] } {
            if {$set_no_synchronizer == 1} {
                set_max_delay -no_synchronizer -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            } else {
                set_max_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            }
            set_min_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -200ns
        } else {
            if {$get_pins == 0} {
                set_net_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -max $max_net_delay
            } else {
                set_net_delay -from [get_pins -compatibility_mode ${from_reg}|q] -to [get_registers ${to_reg}] -max $max_net_delay
            }
            
            # Relax the fitter effort
            if {$set_no_synchronizer == 1} {
                set_max_delay -no_synchronizer -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            } else {
                set_max_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            }
            set_min_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -200ns
        }
    }
  } else {   
        #set skew and min/max delay for EFIFO
        # control skew for bits
        set_max_skew -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -get_skew_value_from_clock_period src_clock_period -skew_value_multiplier 0.8
        # path delay (exception for net delay)
        if { ![string equal "quartus_sta" $::TimeQuestInfo(nameofexecutable)] } {
            set_net_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -max -get_value_from_clock_period dst_clock_period -value_multiplier 0.8
        }
        #relax setup and hold calculation
        if {$set_no_synchronizer == 1} {
            set_max_delay -no_synchronizer -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 100
        } else {
            set_max_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 100
        }
        set_min_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -100		
  }
	
    #set Meta stability for EFIFO 
    if {($set_mstable == 1)} {
		if { ![string equal "quartus_sta" $::TimeQuestInfo(nameofexecutable)] } {
			set_net_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -max -get_value_from_clock_period dst_clock_period -value_multiplier 0.8
		}
    }
}

  
 intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_rx_fully_op" \
                                "*|pma_40g|rx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[*]" \
                                 2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_rx_fully_op" \
                                "*|eth_core|reset_controler|rx_pcs_rst_sync|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_tx_fully_op" \
                                "*|eth_core|reset_controler|sync_tx_fabric_pll_locked|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_tx_fully_op" \
                                "*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_tx_fully_op" \
                                "*|eth_core|reset_controler|sync_tx_digitalreset|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_rx_fully_op" \
                                "*|eth_core|reset_controler|rx_mac_rst_sync|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|hal_top_wrapper_inst|hal_top_ip|one_lane_inst*|one_lane_hal_top_p*|pldif_hal_top_inst|pldif_hal_top|pldif_hal_coreip_inst|gen_sm_ch4_pldif_inst.ch4_pldif_inst|x_std_sm_hssi_pld_chnl_dp_0~pldif_reset_reg" \
                                "*|eth_core|reset_controler|sync_tx_digitalreset|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|hal_top_wrapper_inst|hal_top_ip|one_lane_inst*|one_lane_hal_top_p*|pldif_hal_top_inst|pldif_hal_top|pldif_hal_coreip_inst|gen_sm_ch4_pldif_inst.ch4_pldif_inst|x_std_sm_hssi_pld_chnl_dp_0~pldif_reset_reg" \
                                "*|eth_core|reset_controler|sync_tx_fabric_pll_locked|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_tx_fully_rst" \
                                "*|eth_core|reset_controler|sync_tx_fabric_pll_locked|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_tx_fully_rst" \
                                "*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_tx_fully_rst" \
                                "*|eth_core|reset_controler|sync_tx_digitalreset|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_common_block_rst_done" \
                                "*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_common_block_rst_done" \
                                "*|eth_core|reset_controler|sync_tx_fabric_pll_locked|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_common_block_rst_done" \
                                "*|eth_core|reset_controler|sync_tx_digitalreset|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_rx_fully_op" \
                                "*|pma_40g|rx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_tx_fully_op" \
                                "*|pma_40g|tx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_tx_fully_rst" \
                                "*|pma_40g|tx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0
    intel_eth_e40_constraint_net_delay  "*|pma_40g|native_phy|xcvr_10g_4channel|g1.n.sys[0].n_channel_superset_ip_inst|n_channel_superset_top_wrapper|ncss_common_sip_top_*|src_lane|src_flow_ctrl|src_addr_gen|addr_gen_common_block_rst_done" \
                                "*|pma_40g|tx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[*]" \
                                2.2ns 1 0 0 0 0



set_false_path -from [get_keepers {*|eth_core|pcs|rpcs|am1|locked_r}] -to [get_keepers {*|eth_core|reset_controler|rx_mac_rst_sync|synchronizer_nocut_inst|dreg[0]}]
set_false_path -from [get_keepers {*|eth_core|pcs|rpcs|am1|locked_r}] -to [get_keepers {*|eth_core|reset_controler|rx_mac_rst_sync|synchronizer_nocut_inst|din_s1}]
set_false_path -from [get_keepers {*|eth_core|pcs|rpcs|am1|locked_r}] -to [get_keepers {*|eth_core|reset_controler|rx_mac_rst_sync|synchronizer_nocut_inst|dreg[1]}]

set_false_path -from [get_keepers {*|pma_40g|rx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|rx_fifo_0|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe1a[0]}]
set_false_path -from [get_keepers {*|pma_40g|rx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|rx_fifo_0|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe2a[0]}]
set_false_path -from [get_keepers {*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|tx_fifo_0|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe1a[0]}]
set_false_path -from [get_keepers {*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|tx_fifo_0|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe2a[0]}]

set_false_path -from [get_keepers {*|pma_40g|rx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|rx_fifo_3|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe1a[0]}]
set_false_path -from [get_keepers {*|pma_40g|rx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|rx_fifo_3|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe2a[0]}]
set_false_path -from [get_keepers {*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|tx_fifo_3|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe1a[0]}]
set_false_path -from [get_keepers {*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|tx_fifo_3|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe2a[0]}]


set_false_path -from [get_keepers {*|pma_40g|rx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|rx_fifo_1|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe1a[0]}]
set_false_path -from [get_keepers {*|pma_40g|rx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|rx_fifo_1|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe2a[0]}]
set_false_path -from [get_keepers {*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|tx_fifo_1|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe1a[0]}]
set_false_path -from [get_keepers {*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|tx_fifo_1|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe2a[0]}]


set_false_path -from [get_keepers {*|pma_40g|rx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|rx_fifo_2|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe1a[0]}]
set_false_path -from [get_keepers {*|pma_40g|rx_pll_locked_161_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|rx_fifo_2|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe2a[0]}]
set_false_path -from [get_keepers {*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|tx_fifo_2|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe1a[0]}]
set_false_path -from [get_keepers {*|pma_40g|tx_pll_locked_312p5_sync_0|synchronizer_nocut_inst|dreg[1]}] -to [get_keepers {*|pma_40g|tx_fifo_2|fifo_dc|dcfifo_component|auto_generated|rdaclr|dffe2a[0]}]


