// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



// 40G ethernet top level file
// This module contains the MAC, PCS, PMA, and an optional avalon streaming adapter

/*
               |---------|    128b    |-----|    132b    |-----|   264b    |-----|
256b data <--->| Adapter |<---------->| MAC |<---------->| PCS |<--------->| PMA |<---> 4x serial lanes
               |---------|            |-----|            |-----|           |-----|
*/

`timescale 1 ps / 1 ps
module ex_40g_intel_eth_e40_400_4zmad5a #(
        parameter MODE                  = 1'b0,
        parameter TARGET_CHIP           = 6,
        parameter SIM_EMULATE           = 1'b0,
        parameter SIM_SHORT_AM          = 1'b0,
        parameter SIM_BER_CYCLE_CNT     = 'd39062,  // hi_ber (1/10)*1.25ms counter for SIM
        parameter SYNOPT_LINK_FAULT     = 1'b0,
        parameter DYNAMIC_SKEW          = 1'b0,
        parameter SPEED_CONFIG          = 40,
        parameter EXT_TX_PLL            = 1,
        parameter PHY_REFCLK            = 1,
        parameter SYNOPT_TXCRC_INS      = 1,
        parameter SYNOPT_PREAMBLE_PASS  = 0,
        parameter SYNOPT_MAC_STATS      = 1,
        parameter SYNOPT_READY_LATENCY  = 0,
        parameter SYNOPT_FLOW_CONTROL   = 0,
        parameter SYNOPT_NUMPRIORITY    = 8,
        parameter SYNOPT_STRICT_SOP     = 0,          // UNH SFD compliance feature
        parameter SYNOPT_SYNC_E         = 0,
        parameter enable_cdr_clkout     = 0,
        parameter ENABLE_ADME           = 0,
        parameter ENABLE_JTAG_AVMM      = 0,
        parameter WORDS                 = 2,
        parameter STATUS_CLK_KHZ        = 100000,     // clk_status rate in Mhz
        parameter ENABLE_KR4            = 0, 
        parameter SYNTH_AN              = 0,
        parameter SYNTH_SEQ             = 0,
        parameter LINK_TIMER_KR         = 513,
        parameter AN_TECH               = 8,
        parameter AN_CHAN               = 2,
        parameter AN_PAUSE              = 2,
        parameter SYNTH_LT              = 0,
        parameter BERWIDTH              = 4,
        parameter VMAXRULE              = 30,
        parameter VMINRULE              = 6,
        parameter VODMINRULE            = 14,
        parameter VPOSTRULE             = 25,
        parameter VPRERULE              = 16,
        parameter PREMAINVAL            = 30,
        parameter PREPOSTVAL            = 0,
        parameter PREPREVAL             = 0,
        parameter INITMAINVAL           = 25,
        parameter INITPOSTVAL           = 13,
        parameter INITPREVAL            = 3,
        parameter TRNWTWIDTH            = 8,
        parameter USE_DEBUG_CPU         = 0,
        parameter SYNTH_FEC             = 0,
        parameter CAPABLE_FEC           = 0,
        parameter ENABLE_FEC            = 0,
    parameter RX_PLL_TYPE           = "IOPLL"                
) (
        
        //Avalon ST interface
        output                          clk_rxmac,
        output      [5:0]               l2_rx_error,
        output                          l2_rx_valid,
        output                          l2_rx_startofpacket,
        output                          l2_rx_endofpacket,
        output      [3:0]               l2_rx_empty,
        output      [64*WORDS-1:0]      l2_rx_data,
        output                          clk_txmac,
        input                           l2_tx_error,
        input                           l2_tx_startofpacket,
        input                           l2_tx_endofpacket,
        input                           l2_tx_valid,
        output                          l2_tx_ready,
        input       [3:0]               l2_tx_empty,
        input       [64*WORDS-1:0]      l2_tx_data,
        //Common interface PCS,RECONFIG,ETC 
        output                          tx_lanes_stable,
        output                          rx_pcs_ready,
        output                          rx_hi_ber,
       // input                           clk_ref,
        input                           csr_rst_n,
        input                           tx_rst_n,
        input                           rx_rst_n,
        output                          rx_block_lock,
        output                          rx_am_lock,
        input                           reconfig_clk,
        input                           reconfig_reset,
        input                           reconfig_write,
        input                           reconfig_read,
        input       [20:0]              reconfig_address,
        input       [31:0]              reconfig_writedata,
        output      [31:0]              reconfig_readdata,
        output                          reconfig_waitrequest,
        output                          reconfig_readdatavalid,
        //input [3:0]                 reconfig_byteenable,

        output      [3:0]               tx_serial_p,
        output      [3:0]               tx_serial_n,
        input       [3:0]               rx_serial_p,
        input       [3:0]               rx_serial_n,
        output                          l2_txstatus_valid,
        output      [39:0]              l2_txstatus_data,
        output      [6:0]               l2_txstatus_error,
        output                          l2_rxstatus_valid,
        output      [39:0]              l2_rxstatus_data,
        input       [63:0]              l2_tx_preamble,
        output      [63:0]              l2_rx_preamble,
        input                           clk_status,
        input                           status_write,
        input                           status_read,
        input       [15:0]              status_addr,
        input       [31:0]              status_writedata,
        output      [31:0]              status_readdata,
        output                          status_readdata_valid,
        output                          status_waitrequest,
        output                          clk_rx_recover,
        //Misc. Signals
        output                          remote_fault_status,
        output                          local_fault_status,
        output                          unidirectional_en,
        output                          link_fault_gen_en,
        // Flow control interface
        input  [SYNOPT_NUMPRIORITY-1:0] pause_insert_tx0,
        input  [SYNOPT_NUMPRIORITY-1:0] pause_insert_tx1,
        output [SYNOPT_NUMPRIORITY-1:0] pause_receive_rx,
        
    input  wire                clk_ref_p,                   //                          rx_cdr_refclk_p.clk
        //input  wire                tx_pll_refclk_p,                   //                          tx_pll_refclk_p.clk
        //input  wire                tx_pll_refclk_n,                   //                          tx_pll_refclk_n.clk
        input  wire                i_pma_cu_clk,                   //                          i_pma_cu_clk.i_pma_cu_clk
        input  wire                i_system_pll_lock,                   //                          i_system_pll_lock.i_system_pll_lock
        input  wire                i_system_pll_clk,                   //                          i_system_pll_clk.i_system_pll_clk
       // input  wire                i_tx_reset,                   //                             i_tx_reset.i_tx_reset
        //input  wire                i_rx_reset,                   //                             i_rx_reset.i_rx_reset
      //  output  wire                tx_reset_ack,                   //                             tx_reset_ack.tx_reset_ack
        output  wire                csr_rst_ack_n,                   //                             rx_reset_ack.rx_reset_ack
        output  wire                rx_rst_ack_n,                   //                             rx_reset_ack.rx_reset_ack
        output  wire                tx_rst_ack_n,                   //                             rx_reset_ack.rx_reset_ack

        input  wire [3:0]          i_src_rs_grant,                   //                          i_src_rs_grant.i_src_rs_grant
        output  wire [3:0]          o_src_rs_req,                   //                          o_src_rs_req.o_src_rs_req
      // output wire [3:0]            rx_block_lock,                      //                          rx_block_lock.
       //output wire [3:0]            rx_am_lock,                      //                          rx_am_lock.rx_am_lock
       //output wire [3:0]            local_fault_status,                      //                     local_fault_status.local_fault_status
         output                     cdr_divclk 
        
    );

   


    //------------------------------------------
    //------------- TX MAC PLL  ----------------
    //------------------------------------------
    //wire tx_fabric_pll_locked;

    //------------------------------------------
    //------------- Core Module ----------------
    //------------------------------------------
    wire [2*66*WORDS-1:0]   tx_pcs_data;
    wire [3:0]              tx_pcs_valid;
    wire [3:0]              tx_err_ins;                        //connection with KR4
    
    wire [2*66*WORDS-1:0]   rxpcs_din;
    wire [3:0]              rxpcs_dout_word_locked;
    wire                    rxpcs_dout_deskew_locked;
    wire                    rxpcs_dout_align_locked;
    wire [3:0]              rx_bitslip;
    wire                    inhibit_slip;
    wire                    inhibit_rx_reset;
    wire [3:0]              rx_bitslip_xcvr = inhibit_slip? 4'b0000 : rx_bitslip;
    wire [3:0]              rfifo_rd_en;
    wire                    xcvr_rx_dig_rst_req;
    wire                    pma_reset;
    wire [3:0]              rx_ready;
    wire [3:0]              tx_ready;
    wire [3:0]              rx_data_valid;
    
    wire  [3:0]             rx_clkout;
    wire                    rx_clk;
   
    wire  [3:0]             tx_clkout;
    wire                    tx_clk;

    wire [3:0]              tx_digitalreset;
    wire [3:0]              rx_digitalreset;
    wire                    rx_pcs_pll_locked;

    wire [3:0]              rx_is_lockedtodata;
    wire [3:0]              rx_is_lockedtoref;
    wire [3:0]              rx_analogreset_stat;               //Connection with KR4
    wire [3:0]              tx_analogreset_stat;               //Connection with KR4
    wire [3:0]              rx_digitalreset_stat;              //Connection with KR4
    wire [3:0]              rx_digitalreset_stat_pma,rx_digitalreset_stat_trs_kr;
    wire [3:0]              tx_digitalreset_stat;              //Connection with KR4
    wire [3:0]              tx_fifo_full;
    wire [3:0]              tx_fifo_empty;
    wire [3:0]              tx_fifo_pfull;
    wire [3:0]              tx_fifo_pempty;
    wire [3:0]              rx_fifo_full;
    wire [3:0]              rx_fifo_empty;
    wire [3:0]              rx_fifo_pfull;
    wire [3:0]              rx_fifo_pempty;
    //--Some connections are different for KR mode
    wire [3:0]              tx_coreclk_to_pma,rx_coreclk_to_pma;
    wire [3:0]              tx_coreclkin, rx_coreclkin;
    wire                    reconfig_write_pma, reconfig_write_arb;
    wire                    reconfig_read_pma, reconfig_read_arb;
    wire [3:0]              reconfig_byteenable_pma ;    
    wire [20:0]             reconfig_address_pma;
    wire [12:0]             reconfig_address_arb;              //Connection with KR4
    wire [31:0]             reconfig_writedata_pma;
    wire [31:0]             reconfig_writedata_arb;            //Connection with KR4
    wire                    reconfig_waitrequest_pma, reconfig_waitrequest_arb;
    wire                    rx_set_locktoref; 
    wire                    rx_set_locktodata;                 //Connection with KR4
    wire                    rx_set_locktoref_core;
    wire [3:0]              rx_enh_clr_errblk_count;           //Connection with KR4
    wire [3:0]              rx_enh_blk_lock;                   //Connection with KR4
    wire                    tx_kr4_ready,rx_kr4_ready;         //Connection with KR4
    wire [3:0]              tx_pma_ready,rx_pma_ready;
    wire [3:0]              rx_cdr_locked_kr4;                 //Connection with KR4
    wire                    start_reset_ctrl;                  //Connection with KR4
    wire                    start_reset_ctrl_rx, start_reset_ctrl_tx_d;   //Connection with KR4
    wire                    start_reset_ctrl_tx_aib,start_reset_ctrl_tx_pcs; //Connection with KR4
    wire  [3:0]             rx_cal_busy;                       //Connection with KR4
    wire  [3:0]             tx_cal_busy;                       //Connection with KR4
    wire                    pll_recal_done;

    wire                    rx_pll_recal_done_sync;
    wire                    rx_pcs_pll_locked_sync;
    //reg                     rx_pcs_pll_sync_status_clk; //No logic in top
    wire  [3:0]             rx_transfer_ready,rx_aib_reset, rx_pcs_reset; //Connection with KR4 reset sequencer
    wire  [3:0]             tx_transfer_ready,tx_aib_reset, tx_pcs_reset; //Connection with KR4 reset sequencer
    wire                    reset_aib_ack,reset_pcs_ack;      //Connection with KR4

    // JTAG AVMM Combined CSR Interface
    wire  [15:0]            status_addr_jtag_arb;
    wire                    status_read_jtag_arb;
    wire  [31:0]            status_readdata_jtag_arb;
    wire                    status_write_jtag_arb;
    wire  [31:0]            status_writedata_jtag_arb;
    wire                    status_waitrequest_jtag_arb;
    wire                    status_readdata_valid_jtag_arb;

    // JTAG AVMM Combined Reconfig Interface
    wire  [20:0]            reconfig_address_jtag_arb;
    wire                    reconfig_read_jtag_arb;
    wire  [31:0]            reconfig_readdata_jtag_arb;
    wire                    reconfig_write_jtag_arb;
    wire  [31:0]            reconfig_writedata_jtag_arb;
    wire                    reconfig_waitrequest_jtag_arb;
   wire                reconfig_readdatavalid_jtag_arb;


    wire  [3:0]             tx_ready_etile;
    wire  [3:0]             rx_ready_etile;
    wire  [3:0]             tx_dll_lock;

    //wire  [3:0]             rx_block_lock_w;

wire  [3:0]            local_fault_status_w;                                                
        
//wire  [3:0]            rx_am_lock_w;        
wire  [3:0]      tx_clkout2_w;
wire  [3:0]      rx_clkout2_w;
    
        
        

(* tile_ip_sip,component_type="sip" *)
    alt_e40_synchronizer  rx_pcs_pll_lk_sync (
        .clk    (clk_status),
        .din    (rx_ready_etile),
        .dout   (rx_pcs_pll_locked_sync)
        );

(* tile_ip_sip,component_type="sip" *)
    alt_e40_synchronizer  pll_recal_done_sync (
       .clk    (clk_status),
        .din   (pll_recal_done),
        .dout  (rx_pll_recal_done_sync)
    );

//No logic in top
/*
    always @(posedge clk_status)
       rx_pcs_pll_sync_status_clk <= rx_pcs_pll_locked_sync;
*/
    localparam  kr4_layer = SYNTH_SEQ || SYNTH_FEC ; // SEQ indicates AN/LT ON

//No logic in top
/*
assign tx_digitalreset = 4'b0;
assign rx_digitalreset = 4'b0;
// assign rx_enh_clr_errblk_count = 4'b0;
assign rx_enh_blk_lock = 4'b0;
assign rx_analogreset_stat = 4'b0;
assign tx_analogreset_stat = 4'b0;
// assign rx_digitalreset_stat = 4'b0;
assign tx_digitalreset_stat = 4'b0;
assign rx_is_lockedtoref = 4'b0;
assign rx_cal_busy = 4'b0;
assign tx_cal_busy = 4'b0;
assign rx_digitalreset_stat_pma = 4'b0;
*/

(* tile_ip_sip,component_type="sip" *)
    alt_e40_core_4zmad5a  #(
        .MODE                   (MODE),
        .SIM_EMULATE              (SIM_EMULATE),
        .SIM_SHORT_AM             (SIM_SHORT_AM),
        .SIM_BER_CYCLE_CNT        (SIM_BER_CYCLE_CNT), // hi_ber counter for SIM 
        .SYNOPT_LINK_FAULT        (SYNOPT_LINK_FAULT),
        .DYNAMIC_SKEW             (DYNAMIC_SKEW),
        .TARGET_CHIP              (TARGET_CHIP),
        .EXT_TX_PLL               (EXT_TX_PLL),
        .SYNOPT_TXCRC_INS         (SYNOPT_TXCRC_INS),
        .SYNOPT_PREAMBLE_PASS     (SYNOPT_PREAMBLE_PASS),
        .SYNOPT_MAC_STATS         (SYNOPT_MAC_STATS),
        .SYNOPT_FLOW_CONTROL      (SYNOPT_FLOW_CONTROL),
        .SYNOPT_NUMPRIORITY       (SYNOPT_NUMPRIORITY),
        .SYNOPT_STRICT_SOP        (SYNOPT_STRICT_SOP),
        .SYNOPT_READY_LATENCY     (SYNOPT_READY_LATENCY),
        .ENABLE_KR4               (kr4_layer), 
        .SYNTH_AN                 (SYNTH_AN),
        .SYNTH_LT                 (SYNTH_LT),
        .SYNTH_SEQ                (SYNTH_SEQ),
        .SYNTH_FEC                (SYNTH_FEC),
        .LINK_TIMER_KR            (LINK_TIMER_KR),
        .BERWIDTH                 (BERWIDTH),
        .TRNWTWIDTH               (TRNWTWIDTH),
        .MAINTAPWIDTH             (5),
        .POSTTAPWIDTH             (6),
        .PRETAPWIDTH              (5),
        .VMAXRULE                 (VMAXRULE),
        .VMINRULE                 (VMINRULE),
        .VODMINRULE               (VODMINRULE),
        .VPOSTRULE                (VPOSTRULE),
        .VPRERULE                 (VPRERULE),
        .PREMAINVAL               (PREMAINVAL),
        .PREPOSTVAL               (PREPOSTVAL),
        .PREPREVAL                (PREPREVAL),
        .INITMAINVAL              (INITMAINVAL),
        .INITPOSTVAL              (INITPOSTVAL),
        .INITPREVAL               (INITPREVAL),
        .USE_DEBUG_CPU            (USE_DEBUG_CPU),
        .AN_CHAN                  (AN_CHAN),
        .AN_PAUSE                 (AN_PAUSE),
        .AN_TECH                  (AN_TECH),
        .CAPABLE_FEC              (CAPABLE_FEC),
        .ENABLE_FEC               (ENABLE_FEC),
        .STATUS_CLK_KHZ           (STATUS_CLK_KHZ)
    ) eth_core (
        .tx_lanes_stable          (tx_lanes_stable),
        .avmm_write               (status_write_jtag_arb),
        .avmm_read                (status_read_jtag_arb),
        .avmm_address             (status_addr_jtag_arb),
        .avmm_write_data          (status_writedata_jtag_arb),
        .avmm_read_data           (status_readdata_jtag_arb),
        .avmm_read_data_valid     (status_readdata_valid_jtag_arb),
        .avmm_waitrequest         (status_waitrequest_jtag_arb),
        // Incoming rcfg REQ from TOP
        .reconfig_write_top       (reconfig_write_jtag_arb),
        .reconfig_read_top        (reconfig_read_jtag_arb),
        .reconfig_address_top     (reconfig_address_jtag_arb[12:0]),
        .reconfig_writedata_top   (reconfig_writedata_jtag_arb),
        // Incoming from HSSI 
        .reconfig_readdata_pma    (reconfig_readdata_jtag_arb),
        .reconfig_waitrequest_pma (reconfig_waitrequest_jtag_arb),
        // ARB output to HSSI/TOP
        .reconfig_write_arb       (reconfig_write_arb),
        .reconfig_read_arb        (reconfig_read_arb),
        .reconfig_address_arb     (reconfig_address_arb),
        .reconfig_writedata_arb   (reconfig_writedata_arb),
        .reconfig_waitrequest_arb (reconfig_waitrequest_arb),

        .rx_pempty                (rx_fifo_pempty),
        .rx_pfull                 (rx_fifo_pfull),
        .rx_empty                 (rx_fifo_empty),
        .rx_full                  (rx_fifo_full),
        .rx_set_locktoref         (rx_set_locktoref_core),
        .rx_set_locktodata        (rx_set_locktodata),
        .rx_enh_clr_errblk_count  (rx_enh_clr_errblk_count),
        .rx_enh_blk_lock          (4'b0),
        .tx_pempty                (tx_fifo_pempty),
        .tx_pfull                 (tx_fifo_pfull),
        .tx_empty                 (tx_fifo_empty),
        .tx_full                  (tx_fifo_full),
        .tx_pll_locked            (&tx_dll_lock & (&tx_ready_etile)),
        .rx_seriallpbken          (),
        .tx_digitalreset          (!tx_ready_etile[1]),
        .tx_digitalreset_pma      (1'b0),
        .rx_digitalreset_pma      (1'b0),
        .tx_fabric_pll_locked     ((tx_dll_lock & (&tx_ready_etile))),

        .pause_insert_tx0         (pause_insert_tx0),
        .pause_insert_tx1         (pause_insert_tx1),
        .pause_receive_rx         (pause_receive_rx),

        .l2_tx_error              (l2_tx_error),
        .l2_tx_startofpacket      (l2_tx_startofpacket),
        .l2_tx_endofpacket        (l2_tx_endofpacket),
        .l2_tx_valid              (l2_tx_valid),
        .l2_tx_ready              (l2_tx_ready),
        .l2_tx_empty              (l2_tx_empty),
        .l2_tx_data               (l2_tx_data),
        .l2_txstatus_data         (l2_txstatus_data),
        .l2_txstatus_valid        (l2_txstatus_valid),
        .l2_txstatus_error        (l2_txstatus_error),

        .l2_tx_preamble           (l2_tx_preamble),
        .l2_rx_preamble           (l2_rx_preamble),

        .remote_fault_status      (remote_fault_status),
        .local_fault_status       (local_fault_status),
        .unidirectional_en        (unidirectional_en),
        .link_fault_gen_en        (link_fault_gen_en),

        .l2_rxstatus_data         (l2_rxstatus_data),
        .l2_rxstatus_valid        (l2_rxstatus_valid),

        .l2_rx_error              (l2_rx_error),
        .l2_rx_valid              (l2_rx_valid),
        .l2_rx_startofpacket      (l2_rx_startofpacket),
        .l2_rx_endofpacket        (l2_rx_endofpacket),
        .l2_rx_data               (l2_rx_data),
        .l2_rx_empty              (l2_rx_empty),

    .soft_tx_rst              (soft_tx_rst),
        .inhibit_rx_reset         (inhibit_rx_reset), 
        .inhibit_slip             (inhibit_slip), 
        .tx_rst                   (~tx_rst_n),
        .tx_rst_ack_n             (tx_rst_ack_n),
        .tx_clk                   (tx_clkout2_w[0]),
        .clk_avmm                 (clk_status),
        .tx_pcs_data              (tx_pcs_data),
        .tx_pcs_valid             (tx_pcs_valid),
        .tx_err_ins               (tx_err_ins), 
        //.rx_pcs_pll_locked        (rx_pcs_pll_sync_status_clk),//No logic in top
        .rx_pcs_pll_locked_sync   (rx_pcs_pll_locked_sync),
        .rx_clk                   (rx_clkout2_w[0]),
        .rx_cdr_refclk            (1'b0),
        .rx_rst                   (~rx_rst_n),
        .rx_rst_ack_n             (rx_rst_ack_n),
        .rx_din                   (rxpcs_din),
        .csr_rst_n                (csr_rst_n),
        .rpcs_deskew_locked       (rxpcs_dout_deskew_locked),
        .rpcs_align_locked        (rxpcs_dout_align_locked),
        .rx_is_lockedtodata       (rx_is_lockedtodata),
        .rx_is_lockedtoref        (4'b0),
        .rx_analogreset_stat      (4'b0)  ,
        .tx_analogreset_stat      (4'b0)  ,
        .rx_digitalreset_stat     (4'b0)  ,
        .tx_digitalreset_stat     (4'b0)  ,
        .rx_cal_busy              (4'b0),
        .tx_cal_busy              (4'b0),
        .rpcs_word_locked         (rxpcs_dout_word_locked),
        .rpcs_bit_slip            (rx_bitslip),
        .rpcs_hi_ber              (rx_hi_ber),
        .rpcs_blk_err             (),
        .rfifo_rd_en              (rfifo_rd_en),
        .rx_din_valid             (rx_data_valid),
        .tx_clkout                (tx_clkout),
        .rx_clkout                (rx_clkout),
        .tx_coreclkin             (tx_coreclkin),
        .rx_coreclkin             (rx_coreclkin),

        .tx_digital_reset_status  (~((&tx_dll_lock) & (&tx_ready_etile))),
        .xcvr_rx_dig_rst_out      (xcvr_rx_dig_rst_req),
        .native_phy_reset         (pma_reset),
        .rx_pcs_ready             (rx_pcs_ready),
        .rx_am_lock               (rx_am_lock),
        .rx_ready                 (rx_ready_etile[0]),
        .tx_pma_ready             (tx_ready_etile),
        .tx_kr4_ready             (tx_kr4_ready),
        .rx_pma_ready             (rx_ready_etile),
        .rx_kr4_ready             (rx_kr4_ready),
        .rx_cdr_locked_kr4        (rx_cdr_locked_kr4),
        .start_reset_ctrl         (start_reset_ctrl),
        .start_reset_ctrl_rx      (start_reset_ctrl_rx),
        .start_reset_ctrl_tx_d    (start_reset_ctrl_tx_d),
        .start_reset_ctrl_tx_aib  (start_reset_ctrl_tx_aib),
        .start_reset_ctrl_tx_pcs  (start_reset_ctrl_tx_pcs),
        .reset_aib_ack            (1'b0),
        .reset_pcs_ack            (1'b0),
//PLL calibration done signal for KR
    .rxpll_recal_done         (pll_recal_done)
    );

   //assign rx_block_lock = &rx_block_lock_w;
// assign rx_block_lock = &rxpcs_dout_word_locked; No logic in top
  // assign local_fault_status = &local_fault_status_w;
   //assign rx_am_lock = rx_pcs_ready;


//------------------------------------------------------------------//
// JTAG Master and Arbiter for System Console

//generate
//if (ENABLE_JTAG_AVMM) begin: GEN_JTAG_AVMM

(* tile_ip_sip,component_type="sip" *)
    alt_e40_jtag_avmm #( 
       .ENABLE_JTAG_AVMM (ENABLE_JTAG_AVMM)
      ) e40_jtag_inst(

        // Clock and Resets
        .clk_status                     (clk_status),
        .csr_rst_n                      (csr_rst_n),

        // CSR Interface
        .status_addr                    (status_addr),
        .status_read                    (status_read),
        .status_readdata                (status_readdata),
        .status_write                   (status_write),
        .status_writedata               (status_writedata),
        .status_waitrequest             (status_waitrequest),
        .status_readdata_valid          (status_readdata_valid),

        // Reconfig Interface
        .reconfig_address               (reconfig_address),
        .reconfig_read                  (reconfig_read),
        .reconfig_readdata              (reconfig_readdata),
        .reconfig_write                 (reconfig_write),
        .reconfig_writedata             (reconfig_writedata),
        .reconfig_waitrequest           (reconfig_waitrequest),
        .reconfig_readdatavalid         (reconfig_readdatavalid),
        // Combined CSR Interface
        .status_addr_jtag_arb           (status_addr_jtag_arb),
        .status_read_jtag_arb           (status_read_jtag_arb),
        .status_readdata_jtag_arb       (status_readdata_jtag_arb),
        .status_write_jtag_arb          (status_write_jtag_arb),
        .status_writedata_jtag_arb      (status_writedata_jtag_arb),
        .status_waitrequest_jtag_arb    (status_waitrequest_jtag_arb),
        .status_readdata_valid_jtag_arb (status_readdata_valid_jtag_arb),

        // Combined Reconfig Interface
        .reconfig_address_jtag_arb      (reconfig_address_jtag_arb),
        .reconfig_read_jtag_arb         (reconfig_read_jtag_arb),
        .reconfig_readdata_jtag_arb     (reconfig_readdata_jtag_arb),
        .reconfig_readdatavalid_jtag_arb (reconfig_readdatavalid_jtag_arb),
        .reconfig_write_jtag_arb        (reconfig_write_jtag_arb),
        .reconfig_writedata_jtag_arb    (reconfig_writedata_jtag_arb),
        .reconfig_waitrequest_jtag_arb  (reconfig_waitrequest_jtag_arb)
    );
//end
//else begin: NO_JTAG_AVMM
//    assign  status_addr_jtag_arb        =    status_addr;
//    assign  status_read_jtag_arb        =    status_read;
//    assign  status_readdata             =    status_readdata_jtag_arb;
//    assign  status_write_jtag_arb       =    status_write;
//    assign  status_writedata_jtag_arb   =    status_writedata;
//    assign  status_waitrequest          =    status_waitrequest_jtag_arb;
//    assign  status_readdata_valid       =    status_readdata_valid_jtag_arb;
//
//    assign  reconfig_address_jtag_arb   =    reconfig_address;
//    assign  reconfig_read_jtag_arb      =    reconfig_read;
//    assign  reconfig_readdata           =    reconfig_readdata_jtag_arb;
//    assign  reconfig_write_jtag_arb     =    reconfig_write;
//    assign  reconfig_writedata_jtag_arb =    reconfig_writedata;
//    assign  reconfig_waitrequest        =    reconfig_waitrequest_jtag_arb;
//end
//endgenerate

/*
    //------------------------------------------
    //------------- RX PCS Clock ---------------
    //------------------------------------------
//-------------------------------------------------------------------------------------
// ----------------- Cascding and Calibration details----------------------------------
// on RX side CDR PLL is cascaded to either IOPLL or FPLL
// Here are the details of cascading and the solution we are using for cascading
// In S10 40G Ethernet on RX side we have CDR->PLL cascading, PLL can be IOPLL or FPLL.
// The complication is cascading where refclk of downgrade PLL is coming from CDR, 
// which is not stable after powerup so we need to recalibrate the PLL after powerup. 
// Remember in S10 we do need recalibration of IOPLL which was not the case for A10 IOPLL
// | --------|   |------------------| 
// |  CDR PLL|==>|refclk  IOPLL/FPLL|==> 312.5 Mhz clock (rx_coreclkin and core logic)
// |---------|   |------------------|
//   
// refclk in case of CDR->IOPLL cascading is rx_clkout  = 161.13 Mhz
// refclk in case of CDR->FPLL  cascading is rx_pma_iqtxrxclkout (div40 clk) = 257.8125 Mhz    
// alt_e40_pll_recal.sv implements the recalibration logic
// Recalibration will kicked off at the assertion of either csr_rst_n or 
// soft_csr_rst (aka sys_rst/eio_sys_rst)
// In the logic below pma_reset is used pma_reset is native_phy_reset coming from the reset controller 
// and is asserted when csr_rst_n or sys_rst is asserted.  
// Recalibration sequence will be 
// Initially put CDR in locktoref mode by setting rx_set_locktoref to "1"
// wait CDR to lock to locktoref
// for stable locktoref wait for 20us
// Then start IOPLL/FPLL recalibration
// once recal is done switch CDR to auto mode
// Recalibration done is indicated by pll_recal_done
// Recal happens only once 
//-------------------------------------------------------------------------------------
   wire                    rcfg_write;               // AVMM write
   wire                    rcfg_read;                // AVMM read
   wire [10:0]             rcfg_address;             // AVMM address
   wire [31:0]             rcfg_wrdata;              // AVMM write data
   wire [31:0]             rcfg_rddata;              // AVMM read data
   wire                    rcfg_wtrqst;              // AVMM wait request
   

 // Generate rx_is_lockedtodata_stat signal
 // if rx_is_lockedtodata is "1" for 4 us for L-tile and 5us for H-tile and for Etile it is 180 us.
 // that indicates stable rx_lockedtodata
 // generate rx_is_lockedtodata_stat signal
 // with cnt_lock = 9 and mgmt_clk = 100 mhz (10ns) = 10 *512 = 5.12us
   
   // synchronize rx_is_lockedtodata wrt clk_status
   alt_e40_status_sync 
     #(.WIDTH(1))
   locktodata_sync (
            .clk  (clk_status),
            .din  (&rx_is_lockedtodata),
            .dout (sync_rx_is_lockedtodata)
           );
    
      
  localparam CNTWIDTH = SIM_EMULATE ? 3: 15;  
  localparam MAX_CNT_SYNTH = 15'h4650;
  localparam MAX_CNT_SIM = 3'h7;
  localparam MAX_CNT = SIM_EMULATE ? MAX_CNT_SIM : MAX_CNT_SYNTH;
  reg [CNTWIDTH-1:0] cnt_lock;
  reg                cnt_max;

   // Counter for counting 5us for rx_lockedtodata
   always @(posedge clk_status or posedge pma_reset)
     begin
    if (pma_reset) 
      begin
         cnt_max  <= 1'b0;
         cnt_lock <= {CNTWIDTH{1'b0}};
      end
    else
      begin
           cnt_max <= (cnt_lock == MAX_CNT) ? 1'b1 : 1'b0;
         if (~sync_rx_is_lockedtodata)      
           cnt_lock <= {CNTWIDTH{1'b0}};
         else if (cnt_lock == MAX_CNT)  
           cnt_lock <= cnt_lock;
           else
           cnt_lock <= cnt_lock + 1'b1;
      end // else: !if(mgmt_reset)
     end // always @ (posedge mgmt_clk or posedge mgmt_reset)
   
   assign rx_is_lockedtodata_stat = (cnt_max) ? 1'b1 : 1'b0;
   wire recal_busy;
   wire fpll_cal_busy;
   
   assign recal_busy = (RX_PLL_TYPE == "IOPLL") ? 1'b0 : fpll_cal_busy;

         
// PLL recalibration module to recalibrate IOPLL/FPLL
// 

   alt_e40_pll_recal 
     #(
       .RX_PLL_TYPE(RX_PLL_TYPE)
       ) recal_rxpll 
       (
    .mgmt_clk  (clk_status),
        .mgmt_reset(pma_reset),
        .cal_busy    (recal_busy),
        .rx_is_lockedtoref (1'b1),
    .rx_set_locktoref_core (rx_set_locktoref_core),               
    .rx_set_locktoref (rx_set_locktoref),
        .recal_is_done      (pll_recal_done),

        .rcfg_write(rcfg_write),
        .rcfg_read (rcfg_read),
        .rcfg_address(rcfg_address),
        .rcfg_wrdata (rcfg_wrdata),
        .rcfg_rddata (rcfg_rddata),
        .rcfg_wtrqst (rcfg_wtrqst)
    );
   
//  RX PLL
  generate
     wire [29:0] reconfig_to_iopll;//   reconfig_to_pll to IOPLL   
     wire        rx_pll_reset = (pll_recal_done) ? ~rx_is_lockedtodata_stat : 1'b1;
     assign rcfg_rddata = 32'd0;

   // iopll_rx rx_pll 
     // (
      // .rst        (rx_pll_reset),
      // .refclk     (rx_clkout[0]),         // 161.13 MHz
      // .locked     (rx_pcs_pll_locked),
      // .outclk_0   (rx_clk),                // 312.5  MHz
      // .permit_cal (~rx_pll_reset)
      // );

   //iopll_tx tx_pll_etile 
     // (
      // .rst        (~(&tx_pma_ready & ~pma_reset)),
      // .refclk     (tx_clkout[0]),         // 161.13 MHz
      // .locked     (tx_fabric_pll_locked),
      // .outclk_0   (tx_clk),                // 312.5  MHz
      // .permit_cal ((&tx_pma_ready & ~pma_reset))
     // );
     
 
    //iopll_reconfig  iopll_reconfig
      //(
       //.mgmt_waitrequest    (rcfg_wtrqst),
      // .mgmt_write          (rcfg_write),
       //.mgmt_read           (rcfg_read),
       //.mgmt_writedata      (rcfg_wrdata[7:0]),
       //.mgmt_readdata       (),
       //.mgmt_address        (rcfg_address[9:0]),
       //.mgmt_clk            (clk_status),
       //.mgmt_reset          (pma_reset),
      // .reconfig_from_pll   (11'b0),
      // .reconfig_to_pll     (reconfig_to_iopll)    
     // );
     
   endgenerate
*/

    //assign clk_rxmac = rx_clkout2_w[0];
    //assign clk_txmac = tx_clkout2_w[0];
    //------------------------------------------
    //-------- PMA and reset controller --------
    //------------------------------------------
    // .reset --> will initiate complete reset sequence 
    // .pll_locked --> if down, reset controller will assert TX D reset
    // .rx_cal_busy --> if up, reset controller will assert Rx A/D reset

    wire  [3:0] digital_rx_ready;  //Connection with KR4
/*
   generate 
    if (kr4_layer) begin : KR_CONNECTIONS
       assign tx_coreclk_to_pma             = tx_coreclkin;
       assign rx_coreclk_to_pma             = rx_coreclkin;
       assign reconfig_write_pma            = reconfig_write_arb;
       assign reconfig_read_pma             = reconfig_read_arb;
       assign reconfig_address_pma          = reconfig_address_arb;
       assign reconfig_writedata_pma        = reconfig_writedata_arb;
       assign reconfig_waitrequest_jtag_arb = reconfig_waitrequest_arb;
       assign tx_ready                      = {4{tx_kr4_ready}}; 
       assign rx_ready                      = {4{rx_kr4_ready}}; 
       assign digital_rx_ready              = xcvr_rx_dig_rst_req & !inhibit_rx_reset? 4'b0000 : rx_cdr_locked_kr4;
    end else begin : NONKR_CONNECTIONS
       assign tx_coreclk_to_pma             = {4{tx_clkout2_w[0]}};
       assign rx_coreclk_to_pma             = {4{rx_clkout2_w[0]}};
       assign reconfig_write_pma            = reconfig_write_jtag_arb;
       assign reconfig_read_pma             = reconfig_read_jtag_arb;
       assign reconfig_address_pma          = reconfig_address_jtag_arb;
       assign reconfig_writedata_pma        = reconfig_writedata_jtag_arb;
       assign reconfig_waitrequest_jtag_arb = reconfig_waitrequest_pma;
       assign tx_ready                      = tx_ready_etile; 
       assign rx_ready                      = rx_ready_etile; 
       assign digital_rx_ready              = xcvr_rx_dig_rst_req & !inhibit_rx_reset? 4'b0000 : rx_is_lockedtodata;
    end 
   endgenerate
 */   

//assign clk_rx_recover = rx_clkout2_w[0];

/*
// KR4 TRS is needed only for KR4 NOT non-KR4-FEC
generate 
  if (SYNTH_SEQ) begin : KR4_TRS
//===================================================================
//=== Instantiate Reset Sequencer here ==============================
//===================================================================
    alt_e40_kr4_reset_sequencer_top alt_e40_kr4_reset_sequencer_top_inst
    (   
      .clk_in             (reconfig_clk     ), // used for CDC
      .reset              ( pma_reset       ),
      .release_aib_first  (1'b1             ),
      .reset_in_tx        (4'b0  ),
      .reset_in_rx        (4'b0  ),
      .start_reset_aib    (start_reset_ctrl_tx_aib),
      .start_reset_pcs    (start_reset_ctrl_tx_pcs),
      .reset_aib_ack      (reset_aib_ack    ),
      .reset_pcs_ack      (reset_pcs_ack    ),
      .rx_transfer_ready  (rx_transfer_ready),
      .tx_transfer_ready  (tx_transfer_ready),
      .rx_aib_reset       (rx_aib_reset     ),
      .rx_pcs_reset       (rx_pcs_reset     ),
      .tx_aib_reset       (tx_aib_reset     ),
      .tx_pcs_reset       (tx_pcs_reset     ),
      .rx_reset_out       (rx_digitalreset_stat_trs_kr),
      .tx_reset_out       (4'b0)
//===================================================================
//=== End of Reset Sequencer here ==============================
//===================================================================
    );
       assign rx_digitalreset_stat = rx_digitalreset_stat_trs_kr ;  
  end else begin : NO_KR4_TRS
           assign rx_digitalreset_stat = 4'b0;
       assign rx_aib_reset         = 4'b0;
       assign tx_aib_reset         = 4'b0;
       assign rx_pcs_reset         = 4'b0;
       assign tx_pcs_reset         = 4'b0;
       assign reset_aib_ack        = 1'b0;
       assign reset_pcs_ack        = 1'b0;
  end
endgenerate
*/

//wire  [3:0]      tx_coreclkin_lane;
//wire  [3:0]      rx_coreclkin_lane;
wire tx_reset_ack_w ;
wire rx_reset_ack_w ;

/*
assign csr_rst_ack_n = ~(rx_reset_ack_w & tx_reset_ack_w);

assign tx_coreclkin_lane[0] = tx_clkout[0];
assign tx_coreclkin_lane[1] = tx_clkout[0];
assign tx_coreclkin_lane[2] = tx_clkout[0];
assign tx_coreclkin_lane[3] = tx_clkout[0];

assign rx_coreclkin_lane[0] = tx_clkout[0];
assign rx_coreclkin_lane[1] = tx_clkout[0];
assign rx_coreclkin_lane[2] = tx_clkout[0];
assign rx_coreclkin_lane[3] = tx_clkout[0];

reg  tx_rst_ack_n_w1;

reg  tx_rst_ack_n_w2;
reg  tx_rst_ack_n_w3;
reg  tx_rst_ack_n_w4;


reg  rx_rst_ack_n_w1;

reg  rx_rst_ack_n_w2;
reg  rx_rst_ack_n_w3;
reg  rx_rst_ack_n_w4;
always @(posedge tx_clkout2_w[0]) begin
            tx_rst_ack_n_w1 <= tx_rst_n;
           tx_rst_ack_n_w2 <= tx_rst_ack_n_w1;
           tx_rst_ack_n_w3 <= tx_rst_ack_n_w2;
           tx_rst_ack_n_w4 <= tx_rst_ack_n_w3;
           tx_rst_ack_n <= tx_rst_ack_n_w4;
    end


always @(posedge rx_clkout2_w[0]) begin
            rx_rst_ack_n_w1 <= rx_rst_n;
           rx_rst_ack_n_w2 <= rx_rst_ack_n_w1;
           rx_rst_ack_n_w3 <= rx_rst_ack_n_w2;
           rx_rst_ack_n_w4 <= rx_rst_ack_n_w3;
           rx_rst_ack_n <= rx_rst_ack_n_w4;
    end
*/


   alt_e40_pma_4zmad5a  # (

.SYNOPT_SYNC_E     ( SYNOPT_SYNC_E),
.enable_cdr_clkout ( enable_cdr_clkout )
)pma_40g (
    .tx_ready                    (tx_ready_etile),            
    .rx_ready                    (rx_ready_etile),            
    //.tx_pma_ready                (tx_pma_ready),        
    //.rx_pma_ready                (rx_pma_ready),        
    .tx_serial_data              (tx_serial_p),      
    .tx_serial_data_n            (tx_serial_n),    
    .rx_serial_data              (rx_serial_p),      
    .rx_serial_data_n            (rx_serial_n),    
    //.pll_refclk0                 (clk_ref),         
    .rx_is_lockedtodata          (rx_is_lockedtodata),  
  //  .rx_pmaif_bitslip            (rx_bitslip_xcvr),    
   // .tx_coreclkin                (tx_coreclkin_lane),       
   // .rx_coreclkin                (rx_coreclkin_lane),        
    .tx_clkout                   (tx_clkout),           
    .rx_clkout                   (rx_clkout), 
     .tx_clkout2                   (tx_clkout2_w),           
    .rx_clkout2                   (rx_clkout2_w),
    .clk_rxmac                    (clk_rxmac),
    .clk_txmac                     (clk_txmac),
    .o_rx_cdr_divclk             (cdr_divclk),
     
   // .tx_fifo_full                (tx_fifo_full),        
    //.tx_fifo_empty               (tx_fifo_empty),       
    //.tx_fifo_pfull               (tx_fifo_pfull),       
    //.tx_fifo_pempty              (tx_fifo_pempty),      
    //.rx_fifo_full                (rx_fifo_full),        
    .rx_fifo_empty               (rx_fifo_empty),       
    //.rx_fifo_pfull               (rx_fifo_pfull),       
    //.rx_fifo_pempty              (rx_fifo_pempty),      
    //.rx_fifo_rd_en               ( ),       
    .reconfig_clk                (reconfig_clk),        
    .reconfig_reset              (reconfig_reset),      
    .reconfig_write              (reconfig_write_jtag_arb),      
    .reconfig_read               (reconfig_read_jtag_arb),       
    .reconfig_address            (reconfig_address_jtag_arb),    
    .reconfig_writedata          (reconfig_writedata_jtag_arb),  
    .reconfig_readdata           (reconfig_readdata_jtag_arb),   
    .reconfig_waitrequest        (reconfig_waitrequest_jtag_arb),
    .reconfig_readdatavalid (reconfig_readdatavalid_jtag_arb),  //  output,   width = 1,                     .readdatavalid
   .reconfig_byteenable    (4'hf),    //   input,   width = 4,                     .byteenable
    
    
        .rx_cdr_refclk_p      (clk_ref_p),      //   input,    width = 1,      i_rx_cdr_refclk_p.clk
        .tx_pll_refclk_p      (clk_ref_p),      //   input,    width = 1,      i_tx_pll_refclk_p.clk
        .i_pma_cu_clk           (i_pma_cu_clk),           //   input,    width = 1,           i_pma_cu_clk.pma_cu_clk
        .i_system_pll_clk       (i_system_pll_clk),       //   input,    width = 1,       i_system_pll_clk.clk
        .i_system_pll_lock      (i_system_pll_lock),      //   input,    width = 1,      i_system_pll_lock.system_pll_lock
        .i_tx_reset             ({4{pma_reset}}),             //   input,    width = 1,             i_tx_reset.tx_reset
        .i_rx_reset             ({4{pma_reset}}),             //   input,    width = 1,             i_rx_reset.rx_reset
        .tx_reset_ack         (tx_reset_ack_w),         //  output,    width = 1,         o_tx_reset_ack.tx_reset_ack
        .rx_reset_ack         (rx_reset_ack_w),         //  output,    width = 1,         o_rx_reset_ack.rx_reset_ack
      .csr_rst_ack_n        (csr_rst_ack_n),
        .i_src_rs_grant         (i_src_rs_grant),         //   input,    width = 4,         i_src_rs_grant.src_rs_grant
        .o_src_rs_req           (o_src_rs_req),           //  output,    width = 4,           o_src_rs_req.src_rs_req
        .rx_block_lock_w        (rx_block_lock),        //  output,    width = 1,        o_rx_block_lock.o_rx_block_lock
        //.rx_am_lock           (rx_am_lock_w),           //  output,    width = 4,           o_rx_am_lock.o_rx_am_lock
        .local_fault_status   (local_fault_status_w),   //  output,    width = 4,   o_local_fault_status.o_local_fault_status
        
    .tx_pcs_valid_in             (tx_pcs_valid),
    .tx_pcs_data_in              (tx_pcs_data),
    .rxpcs_din_out               (rxpcs_din),
    .tx_clk                      (tx_clkout2_w[0]),
    .rx_clk                      (rx_clkout2_w[0]),
    .rx_data_valid_out           (rx_data_valid),
    .tx_dll_lock                 (tx_dll_lock),
   // .reset                       ({4{pma_reset}}),
    .tx_pll_locked               (& tx_ready_etile),
    .clk_rx_recover               (clk_rx_recover ),
    .rx_pll_locked               (& rx_ready_etile)
);







endmodule 




