// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1 ps / 1 ps

(* altera_attribute = "-name message_disable 13410"*)

module eth_ex_40g #(
    parameter SIM_HURRY = 1'b0,
    parameter SIM_EMULATE = 1'b0
) (
    input       clk50,
    //input       cpu_resetn,

    output      qsfp_rstn,
    output      qsfp_lowpwr,

    input wire  clk_ref_p,
    input wire  [3:0] rx_serial_p,
    input wire  [3:0] rx_serial_n,
    output wire [3:0] tx_serial_p,
    output wire [3:0] tx_serial_n,
//    output wire [9:0] user_io,
      ///////// SI5391B /////////
      output             SI5391B_OE_n,
      output             SI5391B_RST_n
  );

// config SI5391B to make sure QSFP_REFCLK_p/n output 156.25MHz 
assign SI5391B_OE_n = 1'bZ;
assign SI5391B_RST_n = 1'bZ;  
  
    assign qsfp_rstn    = 1'b1;
    assign qsfp_lowpwr  = 1'b0;

    reg [9:0] clk50_cnt = 10'b0;
    reg [9:0] clk100_cnt = 10'b0;
    reg [9:0] clk_txmac_cnt = 10'b0;
    reg [9:0] clk_rxmac_cnt = 10'b0;

    localparam DEVICE_FAMILY = "Agilex 5";
    localparam WORDS = 2;
    localparam WIDTH = 64;
    localparam SIM_NO_TEMP_SENSE = 1'b1;

    wire sys_pll_locked;
    wire clk100 = clk50;
    wire         i_system_pll_lock;
    wire         i_system_pll_clk;
    wire [3:0]   i_src_rs_grant;
    wire [3:0]   o_src_rs_req;
    wire         i_pma_cu_clk;
   wire reconfig_readdatavalid;

/*
    alt_e40_sys_pll sys_pll (
        .rst        (~cpu_resetn),
        .refclk     (clk50),
        .locked     (sys_pll_locked),
        .outclk_0   (clk100)
    );

    wire sys_pll_locked_stretched;
    alt_e40_locklag5 rd0 (
        .clk    (clk100),
        .din    (sys_pll_locked),
        .dout   (sys_pll_locked_stretched)
    );

    defparam rd0 .SIM_HURRY = SIM_HURRY;
    defparam rd0 .SIM_EMULATE = SIM_EMULATE;
*/
  //Instantiate S10 IP reset:
    wire ninit_done ;
    reset_ip reset (
	.ninit_done (ninit_done)
    );
    wire    arst;
    wire    [14:0] source;
    assign  arst = source[0]  ;

// changed from 16 bit to 22 bit to accomodate PMA XCVR reconfig space
    wire [21:0]     status_addr;
    wire            status_read,status_write,status_readdata_valid_eth, status_waitrequest_eth;
    wire [31:0]     status_readdata_eth, status_writedata;
    wire            status_read_timeout = 0;
    wire            clk_status = clk100;
    wire            clk_txmac;
    wire            clk_rxmac;

    wire [127:0]    l2_tx_data;
    wire [3:0]      l2_tx_empty;
    wire            l2_tx_endofpacket;
    wire            l2_tx_ready;
    wire            l2_tx_startofpacket;
    wire            l2_tx_valid;
    wire [127:0]    l2_rx_data;
    wire [3:0]      l2_rx_empty;
    wire            l2_rx_endofpacket;
    wire            l2_rx_startofpacket;
    wire            l2_rx_valid;
    wire [5:0]      l2_rx_error;
  wire           reconfig_readdatavalid;
    // map reconfig registers to 0x4000-0x7FFF
    wire        reco_waitrequest;
    wire        reco_readdata_valid;
    wire [31:0] reco_readdata;
    wire 	reco_cs;
    reg         status_read_r;
    reg         status_write_r;
    reg  [31:0] status_writedata_r;
// changed from 16 bit to 22 bit to accomodate PMA XCVR reconfig space
    reg  [21:0] status_addr_r;

        wire eth_cs    = ((status_addr_r >= 22'h000000) && (status_addr_r <= 22'h000DFF));
    wire select_waitrequest = eth_cs ? status_waitrequest_eth : (reco_cs ? reco_waitrequest : 1'b0);
    always @(posedge clk_status) begin
        if (arst) begin
            status_read_r <= 0;
            status_write_r <= 0;
            status_writedata_r <= 32'b0;
// changed from 16 bit to 22 bit to accomodate PMA XCVR reconfig space
            status_addr_r <= 22'd0;
        end
        else if (!select_waitrequest || status_read || status_write) begin
            status_read_r <= status_read;
            status_write_r <= status_write;
            status_writedata_r <= status_writedata;
            status_addr_r <= status_addr;
        end
    end
 
// changed from 16 bit to 22 bit to accomodate PMA XCVR reconfig space
   
assign reco_cs = ((status_addr_r >= 22'h008000) && (status_addr_r <= 22'h0FFFFF));

    wire        reco_read  = status_read_r  && reco_cs;
    wire        reco_write = status_write_r && reco_cs;
// changed from 16 bit to 22 bit to accomodate PMA XCVR reconfig space max
//21 bits used for PMA XCVR reconfig address space
    wire [20:0] reco_addr = (reco_read || reco_write) ? status_addr_r[20:0] : 21'd0;
   // assign      reco_readdata_valid = reco_read && !reco_waitrequest;
    assign      reco_readdata_valid = reconfig_readdatavalid;

    wire        status_waitrequest;
    wire        rx_block_lock;
    wire        rx_am_lock;
    wire        rx_pcs_ready;
    wire        tx_lanes_stable;


wire definetest;
`ifdef ALTERA_RESERVED_QIS
 assign definetest = clk50_cnt[5];
`else
 assign definetest = clk_txmac_cnt[5];
`endif


    wire eth_read  = (eth_cs && status_read_r);
    wire eth_write = (eth_cs && status_write_r);
// changed from 16 bit to 22 bit to accomodate PMA XCVR reconfig space max
//only 16 bits used for MAC/PCS ethernet address space
    wire [15:0] eth_addr = eth_cs ? status_addr_r[15:0] : 16'd0;

       ex_40g av_top (
             .i_system_pll_lock      (i_system_pll_lock),
             .i_system_pll_clk       (i_system_pll_clk),
             .i_src_rs_grant         (i_src_rs_grant),
             .o_src_rs_req           (o_src_rs_req),
             .i_pma_cu_clk           (i_pma_cu_clk),
             //.loop_done		        ( source_reset[7]      ),
             .clk_ref_p              (clk_ref_p),
             .csr_rst_n              (~arst),
             .csr_rst_ack_n          (system_status[10]),
             .tx_rst_ack_n           (system_status[8]),
             .rx_rst_ack_n           (system_status[9]),
             .tx_rst_n               (source[1]),
             .rx_rst_n               (source[2]),
             .clk_status             (clk_status),
             //.reset_status           (1'b0),


             .rx_serial_p              (rx_serial_p),
             .rx_serial_n              (rx_serial_n),
             .tx_serial_p              (tx_serial_p),
             .tx_serial_n              (tx_serial_n),

             .status_write           (eth_write),
             .status_read            (eth_read),
             .status_addr            (eth_addr),
             .status_writedata       (status_writedata),
             .status_readdata        (status_readdata_eth),
             .status_readdata_valid  (status_readdata_valid_eth),
             .status_waitrequest     (status_waitrequest_eth),


             .clk_txmac              (clk_txmac),
             .l2_tx_startofpacket    (l2_tx_startofpacket),
             .l2_tx_endofpacket      (l2_tx_endofpacket),
             .l2_tx_valid            (l2_tx_valid),
             .l2_tx_ready            (l2_tx_ready),
             .l2_tx_empty            (l2_tx_empty),
             .l2_tx_data             (l2_tx_data),

             .tx_lanes_stable        (tx_lanes_stable),
             .rx_block_lock          (rx_block_lock),
             .rx_am_lock             (rx_am_lock),
             .rx_pcs_ready           (rx_pcs_ready),
             .clk_rxmac              (clk_rxmac),

             .l2_txstatus_valid      (/* Unused */),    // 1  bits
             .l2_txstatus_data       (/* Unused */),    // 40 bits
             .l2_txstatus_error      (/* Unused */),    // 7  bits
             .l2_rxstatus_valid      (/* Unused */),    // 1  bits
             .l2_rxstatus_data       (/* Unused */),    // 40 bits

             .l2_rx_error            (l2_rx_error),
             .l2_rx_valid            (l2_rx_valid),
             .l2_rx_startofpacket    (l2_rx_startofpacket),
             .l2_rx_endofpacket      (l2_rx_endofpacket),
             .l2_rx_data             (l2_rx_data),
             .l2_rx_empty            (l2_rx_empty),




             .reconfig_clk           (clk_status),
             .reconfig_reset         (arst),
             .reconfig_write         (reco_write),
             .reconfig_read          (reco_read),
             .reconfig_address       (reco_addr),
             .reconfig_writedata     (status_writedata_r[31:0]),
             .reconfig_readdata      (reco_readdata[31:0]),
             .reconfig_readdatavalid (reconfig_readdatavalid),
             .reconfig_waitrequest   (reco_waitrequest)
       );


    wire [31:0] status_readdata_pc;
    wire status_readdata_valid_pc;

    alt_e40_packet_client pc (
        .arst        (arst),
        
        // TX to Ethernet
        .clk_tx                 (clk_txmac),
        .tx_valid               (l2_tx_valid),
        .tx_ack                 (l2_tx_ready),
        .tx_data                (l2_tx_data),
        .tx_start               (l2_tx_startofpacket),
        .tx_end                 (l2_tx_endofpacket),
        .tx_empty               (l2_tx_empty),
        .tx_lanes_stable        (tx_lanes_stable),
        
        // RX from Ethernet
        .clk_rx                 (clk_rxmac),
        .rx_valid               (l2_rx_valid),
        .rx_data                (l2_rx_data),
        .rx_start               (l2_rx_startofpacket),
        .rx_end                 (l2_rx_endofpacket),
        .rx_empty               (l2_rx_empty),
        .rx_error               (l2_rx_error),
        
        .rx_block_lock          (rx_block_lock),
        .rx_am_lock             (rx_am_lock),
        .rx_pcs_ready           (rx_pcs_ready),
        // status register bus
        .clk_status             (clk_status),
        .status_addr            (status_addr[15:0]),
        .status_read            (status_read),
        .status_write           (status_write),
        .status_writedata       (status_writedata),
        .status_readdata        (status_readdata_pc),
        .status_readdata_valid  (status_readdata_valid_pc)
    );
    defparam pc.WORDS = WORDS;
    defparam pc.WIDTH = WIDTH;
    defparam pc.SIM_NO_TEMP_SENSE= SIM_NO_TEMP_SENSE;
    defparam pc.DEVICE_FAMILY = DEVICE_FAMILY;

    // _____________________________________________________________
    // merge status bus
    // _____________________________________________________________

    wire [31:0] status_readdata;
    wire status_readdata_valid;
    reg  [2:0] client_readdata_valid;

    always @(*) begin
        if (status_readdata_valid_pc) client_readdata_valid = 3'b010;
        else if (reco_readdata_valid) client_readdata_valid = 3'b001;
        else client_readdata_valid = {status_readdata_valid_eth|status_read_timeout, 2'b00};
    end

    alt_e40_avmm_combine #(
        .NUM_CLIENTS            (3)
    ) arc (
        .clk                    (clk_status),
        .arst                   (arst),
        .host_read              (status_read),
        .host_readdata          (status_readdata),
        .host_readdata_valid    (status_readdata_valid),
        .host_waitrequest       (status_waitrequest),

        .client_readdata_valid  (client_readdata_valid),
        .client_readdata        ({status_readdata_eth,
                                  status_readdata_pc,
                                  reco_readdata})
    );

    // _______________________________________________________________________________________________________________ 
    // jtag_avalon 
    // _______________________________________________________________________________________________________________
    wire [31:0] av_addr;
// changed from 16 bit to 22 bit to accomodate PMA XCVR reconfig space max
    assign status_addr = av_addr[23:2];
    wire [3:0] byteenable;

    alt_e40_jtag_bridge jtag_bridge (
        .clk_clk              (clk_status),
        .clk_reset_reset      (arst),
        .master_address       (av_addr),
        .master_readdata      (status_readdata),
        .master_read          (status_read),
        .master_write         (status_write),
        .master_writedata     (status_writedata),
        .master_waitrequest   (status_waitrequest),
        .master_readdatavalid (status_readdata_valid),
        .master_byteenable    (byteenable),
        .master_reset_reset   ()
    );



    always @(posedge clk_rxmac) begin
	clk_rxmac_cnt <= clk_rxmac_cnt + 10'd1;
    end

    always @(posedge clk_txmac) begin
	clk_txmac_cnt <= clk_txmac_cnt + 10'd1;
    end

    always @(posedge clk50) begin
	clk50_cnt <= clk50_cnt + 10'd1;
    end

    always @(posedge clk100) begin
	clk100_cnt <= clk100_cnt + 10'd1;
    end


    // Clock dividers
    wire top_bit_clk50;
    div_230 #(
        .NBITS(10)
    ) div_clk50 (
        .clk(clk50),
        .rst(1'b0),
        .top_bit(top_bit_clk50)
    );

    wire top_bit_clk100;
    div_230 #(
        .NBITS(10)
    ) div_clk100 (
        .clk(clk100),
        .rst(1'b0),
        .top_bit(top_bit_clk100)
    );



    wire [14:0] system_status;

    assign system_status[0] = arst;
/*
    assign system_status[1] = top_bit_clkref;
    assign system_status[2] = top_bit_clk50;
    assign system_status[3] = top_bit_clk100;
*/
    assign system_status[1] = clk_txmac_cnt[5];
    assign system_status[2] = clk50_cnt[3];
    assign system_status[3] = clk_rxmac_cnt[5];

    assign system_status[4] = tx_lanes_stable;
    assign system_status[5] = rx_block_lock;
    assign system_status[6] = rx_am_lock;
    assign system_status[7] = rx_pcs_ready;
/*
assign user_io[0]= clk_txmac_cnt[5];
assign user_io[1]= clk_rxmac_cnt[5];
assign user_io[2]= clk50_cnt[5];
assign user_io[3]= clk100_cnt[5];
assign user_io[4]= arst;
assign user_io[5]= definetest;
// The following assignment are not used, they're used to fix warnings for unused output
assign user_io[6]= definetest;
assign user_io[7]= definetest;
assign user_io[8]= definetest;
assign user_io[9]= definetest;
*/
    probe8 prb (
       .source(source),
       .probe(system_status)
    );

    intel_systemclk_gts_sm syspll_inst (
    		.o_pll_lock(i_system_pll_lock),
     		.o_syspll_c0(i_system_pll_clk), //clk output to be connected to phy
     		.i_refclk(clk_ref_p),
     		.i_refclk_ready	(1'b1)
    );
    
    intel_srcss_sm sss_sm (
    		.o_src_rs_grant(i_src_rs_grant),
     		.i_src_rs_priority(1'b0),
     		.i_src_rs_req(o_src_rs_req),
     		.o_pma_cu_clk(i_pma_cu_clk)
    );


    //assign user_led = ~system_status[7:0];
endmodule


