# (C) 2001-2024 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


# (C) 2001-2016 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel MegaCore Function License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


source [file join [file dirname [info script]] "altera/sval_top/main.tcl"]
source [file join [file dirname [info script]] "40g/basic.tcl"]
source [file join [file dirname [info script]] "40g/status.tcl"]
source [file join [file dirname [info script]] "common/issp_reset.tcl"]
source [file join [file dirname [info script]] "parameter.tcl"]
source [file join [file dirname [info script]] "silb.tcl"]

#switch_jtag "*A5E*"
set TEST_ERROR 1

proc run_traffic_test { time_s lb_test } {
    
    global TEST_ERROR

    #Clearing pkt client regs 
    reset_counters
    
    set test_name $lb_test

    set time_ms [expr {$time_s * 1000}]
    stop_pkt_gen
    
    #Clearing MAC stat counters
    clear_all_stats

    start_pkt_gen
    puts "Transmitting packets for $time_s seconds"
    after $time_ms
    stop_pkt_gen
  
   
    set tx_pkt_cnt_pktclient [format %u [get_tx_count_pc]]
    set rx_pkt_cnt_pktclient [format %u [get_rx_count_pc]]
    set err_count [format %u [get_error_count_pc]]
    
   
    puts "------------------ Reading MAC Statistics --------------------"
    chkmac_stats
    


    if { [verify_rxtx_counts] != 0 || [check_packet_errors] != 0 || \
          ($rx_pkt_cnt_pktclient != $tx_pkt_cnt_pktclient) || ($err_count != 0) || \
	  ($rx_pkt_cnt_pktclient != [get_total_pkt_count_rx]) || ($tx_pkt_cnt_pktclient != [get_total_pkt_count_rx])} { 

       puts "---------------------------------------------------------------------"
       puts "${test_name}: FAIL - Traffic Test Failed"
       puts "---------------------------------------------------------------------"
    
    } elseif { $rx_pkt_cnt_pktclient == 0 && $rx_pkt_cnt_pktclient == 0 } {

       puts "---------------------------------------------------------------------"
       puts "${test_name}: FAIL - No Packets Transmitted or Received at Packet Client"
       puts "---------------------------------------------------------------------"

    } else {

       set TEST_ERROR 0
       puts "---------------------------------------------------------------------"
       puts "${test_name}: PASS - Tx and Rx Packets Match"
       puts "---------------------------------------------------------------------"
    }
  

}



proc run_test_elb {} {

  set TOTAL_TEST 1
  set TOTAL_PASS 0
  global TEST_ERROR
  set TEST_ERROR 1
  
  chkphy_status
  run_traffic_test 10 "ELB"

  
  if {$TEST_ERROR == 0} {
     set TOTAL_PASS [expr $TOTAL_PASS + 1]

  } 


  puts "========================================================"
  puts "HW_INFO: Hardware Testing Result"
  puts "========================================================"
  if {$TOTAL_PASS == $TOTAL_TEST} {
    puts "########################### Hardware testing passed! ###########################"
	puts "HW_INFO: Test Pass"
	puts "HW_INFO: Test Result: \[$TOTAL_PASS/$TOTAL_TEST\] \[PASS/TOTAL\]"
  } else {
    puts "########################### Hardware testing failed! Please check the log ###########################"
	puts "HW_INFO: Test Fail"
	puts "HW_INFO: Test Result: \[$TOTAL_PASS/$TOTAL_TEST\] \[PASS/TOTAL\]"
  } 

}

proc run_test_silb {} {

  set TOTAL_TEST 1
  set TOTAL_PASS 0
  global TEST_ERROR
  global SILB_ERROR
  set TEST_ERROR 1

  source silb.tcl

  #Enable SILB
  loop_on
  
  if {$SILB_ERROR == 0} { 
    puts "--------------------------------------"
    puts "Reading PHY status registers" 
    puts "--------------------------------------"
  
    chkphy_status
    run_traffic_test 10 "SILB"

    if {$TEST_ERROR == 0} {
     set TOTAL_PASS [expr $TOTAL_PASS + 1]

    }
   } 

 

  puts "========================================================"
  puts "HW_INFO: Hardware Testing Result"
  puts "========================================================"
  if {$TOTAL_PASS == $TOTAL_TEST} {
    puts "########################### Hardware testing passed! ###########################"
	puts "HW_INFO: Test Pass"
	puts "HW_INFO: Test Result: \[$TOTAL_PASS/$TOTAL_TEST\] \[PASS/TOTAL\]"
  } else {
    puts "########################### Hardware testing failed! Please check the log ###########################"
	puts "HW_INFO: Test Fail"
	puts "HW_INFO: Test Result: \[$TOTAL_PASS/$TOTAL_TEST\] \[PASS/TOTAL\]"
  } 

  puts "----------------------------------------------------------------"
  puts "                   SILB TEST COMPLETED                           "
  puts "----------------------------------------------------------------"

  #Disable SILB
  loop_off


}
