/*
This program demonstrate how to use hps communicate with FPGA through light AXI Bridge.
uses should program the FPGA by GHRD project before executing the program
refer to user manual chapter 7 for details about the demo
*/


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdint.h>
#include <sys/mman.h>

// Agilex 5 HPS Register Map
// https://www.intel.com/content/www/us/en/programmable/hps/agilex5/topics/index_frames.html

//////// HPS2FPGA LW Bridge ///////////
// lwhps2fpga_512M	
// https://www.intel.com/content/www/us/en/programmable/hps/agilex5/topics/index_frames.html
#define LWHPS2FPGA_BASE ( 0x20000000 )  //LWHPS2FPGA_memory
#define LWHPS2FPGA_SPAN ( 0x20000000 )  // 512MB
#define LWHPS2FPGA_MASK ( LWHPS2FPGA_SPAN - 1 )

#define LED_PIO_BASE 0x10080//// GHRD/peripheral_subsys/peripheral_subsys/peripheral_subsys.html
#define LED_PIO_DATA_WIDTH	3

//////// HPS2FPGA Bridge ///////////
//hps2fpga_1G. not used in this demo
#define HPS2FPGA_BASE ( 0x40000000 )  //LWHPS2FPGA_memory
#define HPS2FPGA_SPAN ( 0x80000000 )
#define HPS2FPGA_MASK ( HPS2FPGA_SPAN - 1 )

// GHRD/ip/qsys_top/ocm/ocm.html
// not used in this demo
#define ON_MEM_BASE 0x00000000 
#define ON_MEM_SIZE 262144 // bytes

int main() {

	void *virtual_base;
	int fd;
	int loop_count;
	int led_direction;
	int led_mask;
	void *h2p_lw_led_addr;


	// map the address space for the LED registers into user space so we can interact with them.
	// we'll actually map in the entire CSR span of the HPS since we want to access various registers within that span

	if( ( fd = open( "/dev/mem", ( O_RDWR | O_SYNC ) ) ) == -1 ) {
		printf( "ERROR: could not open \"/dev/mem\"...\n" );
		return( 1 );
	}

	virtual_base = mmap( NULL, LWHPS2FPGA_SPAN, ( PROT_READ | PROT_WRITE ), MAP_SHARED, fd, LWHPS2FPGA_BASE );

	if( virtual_base == MAP_FAILED ) {
		printf( "ERROR: mmap() failed...\n" );
		close( fd );
		return( 1 );
	}

	h2p_lw_led_addr=virtual_base + ( ( unsigned long  )( LED_PIO_BASE ) & ( unsigned long)( LWHPS2FPGA_MASK ) );
	

	// toggle the LEDs a bit

	loop_count = 0;
	led_mask = 0x01;
	led_direction = 0; // 0: left to right direction
	while( loop_count < 60 ) {
		
		// control led, add ~ because the led is low-active
		*(uint32_t *)h2p_lw_led_addr = ~led_mask; 

		// wait 200ms
		usleep( 200*1000 );
		
		// update led mask
		if (led_direction == 0){
			led_mask <<= 1;
			if (led_mask == (0x01 << (LED_PIO_DATA_WIDTH-1)))
				 led_direction = 1;
		}else{
			led_mask >>= 1;
			if (led_mask == 0x01){ 
				led_direction = 0;
				loop_count++;
			}
		}
		
	} // while
	

	// clean up our memory mapping and exit
	
	if( munmap( virtual_base, LWHPS2FPGA_SPAN ) != 0 ) {
		printf( "ERROR: munmap() failed...\n" );
		close( fd );
		return( 1 );
	}

	close( fd );

	return( 0 );
}
